"""
统计数据存储模块

负责统计数据的持久化存储和读取
"""

import json
import os
from datetime import datetime, timedelta
from pathlib import Path
from typing import Dict, List, Optional, Any
from collections import defaultdict
import sys
import time
import uuid


class StatsStorage:
    """统计数据存储类"""

    def __init__(self, storage_dir: Optional[str] = None):
        """
        初始化存储

        Args:
            storage_dir: 存储目录路径，默认为 ~/.jarvis/stats
        """
        if storage_dir is None:
            storage_dir = os.path.expanduser("~/.jarvis/stats")

        self.storage_dir = Path(storage_dir)
        self.storage_dir.mkdir(parents=True, exist_ok=True)

        # 数据目录路径
        self.data_dir = self.storage_dir / "data"
        self.data_dir.mkdir(exist_ok=True)

        # 元数据文件路径
        self.meta_file = self.storage_dir / "stats_meta.json"

        # 初始化元数据
        self._init_metadata()

    def _init_metadata(self):
        """初始化元数据"""
        if not self.meta_file.exists():
            meta = {
                "version": "1.0.0",
                "created_at": datetime.now().isoformat(),
                "metrics": {},  # 存储各个指标的元信息
            }
            self._save_json(self.meta_file, meta)

    def _get_data_file(self, date_str: str) -> Path:
        """获取指定日期的数据文件路径"""
        return self.data_dir / f"stats_{date_str}.json"

    def _load_json(self, filepath: Path) -> Dict:
        """加载JSON文件"""
        if not filepath.exists():
            return {}

        # 重试机制处理并发访问
        max_retries = 3
        for attempt in range(max_retries):
            try:
                with open(filepath, "r", encoding="utf-8") as f:
                    data = json.load(f)
                    return data
            except (json.JSONDecodeError, IOError):
                if attempt < max_retries - 1:
                    time.sleep(0.1 * (attempt + 1))  # 递增延迟
                    continue
                return {}
        return {}

    def _save_json(self, filepath: Path, data: Dict):
        """保存JSON文件"""
        # 使用临时文件+重命名的原子操作来避免并发写入问题
        # 使用唯一的临时文件名避免并发冲突
        temp_suffix = f".tmp.{uuid.uuid4().hex[:8]}"
        temp_filepath = filepath.with_suffix(temp_suffix)
        max_retries = 3

        for attempt in range(max_retries):
            try:
                # 先写入临时文件
                with open(temp_filepath, "w", encoding="utf-8") as f:
                    json.dump(data, f, ensure_ascii=False, indent=2)

                # Windows上需要先删除目标文件（如果存在）
                if sys.platform == "win32" and filepath.exists():
                    filepath.unlink()

                # 原子性重命名
                temp_filepath.rename(filepath)
                return

            except Exception as e:
                if attempt < max_retries - 1:
                    time.sleep(0.1 * (attempt + 1))  # 递增延迟
                    continue
                # 清理临时文件
                if temp_filepath.exists():
                    try:
                        temp_filepath.unlink()
                    except OSError:
                        pass
                raise RuntimeError(f"保存数据失败: {e}") from e

    def add_metric(
        self,
        metric_name: str,
        value: float,
        unit: Optional[str] = None,
        timestamp: Optional[datetime] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        """
        添加统计数据

        Args:
            metric_name: 指标名称
            value: 指标值
            unit: 单位
            timestamp: 时间戳，默认为当前时间
            tags: 标签字典，用于数据分类
        """
        if timestamp is None:
            timestamp = datetime.now()

        # 更新元数据
        meta = self._load_json(self.meta_file)
        if metric_name not in meta["metrics"]:
            meta["metrics"][metric_name] = {
                "unit": unit,
                "created_at": timestamp.isoformat(),
                "last_updated": timestamp.isoformat(),
            }
        else:
            meta["metrics"][metric_name]["last_updated"] = timestamp.isoformat()
            if unit and not meta["metrics"][metric_name].get("unit"):
                meta["metrics"][metric_name]["unit"] = unit
        self._save_json(self.meta_file, meta)

        # 获取日期对应的数据文件
        date_key = timestamp.strftime("%Y-%m-%d")
        hour_key = timestamp.strftime("%H")
        date_file = self._get_data_file(date_key)

        # 加载日期文件的数据
        data = self._load_json(date_file)

        # 组织数据结构：metric_name -> hour -> records
        if metric_name not in data:
            data[metric_name] = {}

        if hour_key not in data[metric_name]:
            data[metric_name][hour_key] = []

        # 添加数据记录
        record = {
            "timestamp": timestamp.isoformat(),
            "value": value,
            "tags": tags or {},
        }
        data[metric_name][hour_key].append(record)

        # 保存数据到日期文件
        self._save_json(date_file, data)

    def get_metrics(
        self,
        metric_name: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[Dict]:
        """
        获取指定时间范围的统计数据

        Args:
            metric_name: 指标名称
            start_time: 开始时间
            end_time: 结束时间
            tags: 过滤标签

        Returns:
            数据记录列表
        """
        # 默认时间范围
        if end_time is None:
            end_time = datetime.now()
        if start_time is None:
            start_time = end_time - timedelta(days=7)  # 默认最近7天

        results = []

        # 遍历日期
        current_date = start_time.date()
        end_date = end_time.date()

        while current_date <= end_date:
            date_key = current_date.strftime("%Y-%m-%d")
            date_file = self._get_data_file(date_key)

            # 如果日期文件不存在，跳过
            if not date_file.exists():
                current_date += timedelta(days=1)
                continue

            # 加载日期文件的数据
            data = self._load_json(date_file)

            if metric_name in data:
                for hour_key, records in data[metric_name].items():
                    for record in records:
                        record_time = datetime.fromisoformat(record["timestamp"])

                        # 检查时间范围
                        if start_time <= record_time <= end_time:
                            # 检查标签过滤
                            if tags:
                                record_tags = record.get("tags", {})
                                if all(
                                    record_tags.get(k) == v for k, v in tags.items()
                                ):
                                    results.append(record)
                            else:
                                results.append(record)

            current_date += timedelta(days=1)

        # 按时间排序
        results.sort(key=lambda x: x["timestamp"])

        return results

    def get_metric_info(self, metric_name: str) -> Optional[Dict]:
        """获取指标元信息"""
        meta = self._load_json(self.meta_file)
        return meta.get("metrics", {}).get(metric_name)

    def list_metrics(self) -> List[str]:
        """列出所有指标"""
        # 从元数据文件获取指标
        meta = self._load_json(self.meta_file)
        metrics_from_meta = set(meta.get("metrics", {}).keys())
        
        # 扫描所有数据文件获取实际存在的指标
        metrics_from_data: set[str] = set()
        for data_file in self.data_dir.glob("stats_*.json"):
            try:
                data = self._load_json(data_file)
                metrics_from_data.update(data.keys())
            except (json.JSONDecodeError, OSError):
                # 忽略无法读取的文件
                continue
        
        # 合并两个来源的指标并返回排序后的列表
        all_metrics = metrics_from_meta.union(metrics_from_data)
        return sorted(list(all_metrics))

    def aggregate_metrics(
        self,
        metric_name: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        aggregation: str = "hourly",
        tags: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Dict[str, Any]]:
        """
        聚合统计数据

        Args:
            metric_name: 指标名称
            start_time: 开始时间
            end_time: 结束时间
            aggregation: 聚合方式 (hourly, daily)
            tags: 过滤标签

        Returns:
            聚合后的数据字典
        """
        records = self.get_metrics(metric_name, start_time, end_time, tags)

        if not records:
            return {}

        # 聚合数据
        aggregated: Dict[str, Dict[str, Any]] = defaultdict(
            lambda: {
                "count": 0,
                "sum": 0,
                "min": float("inf"),
                "max": float("-inf"),
                "values": [],
            }
        )

        for record in records:
            timestamp = datetime.fromisoformat(record["timestamp"])
            value = record["value"]

            if aggregation == "hourly":
                key = timestamp.strftime("%Y-%m-%d %H:00")
            elif aggregation == "daily":
                key = timestamp.strftime("%Y-%m-%d")
            else:
                key = timestamp.strftime("%Y-%m-%d %H:00")

            aggregated[key]["count"] += 1
            aggregated[key]["sum"] += value
            aggregated[key]["min"] = min(aggregated[key]["min"], value)
            aggregated[key]["max"] = max(aggregated[key]["max"], value)
            aggregated[key]["values"].append(value)

        # 计算平均值
        result = {}
        for key, stats in aggregated.items():
            result[key] = {
                "count": stats["count"],
                "sum": stats["sum"],
                "min": stats["min"],
                "max": stats["max"],
                "avg": stats["sum"] / stats["count"] if stats["count"] > 0 else 0,
            }

        return result

    def delete_metric(self, metric_name: str) -> bool:
        """
        删除指定的指标及其所有数据
        
        Args:
            metric_name: 要删除的指标名称
            
        Returns:
            True 如果成功删除，False 如果指标不存在
        """
        # 检查指标是否存在
        meta = self._load_json(self.meta_file)
        if metric_name not in meta.get("metrics", {}):
            return False
            
        # 从元数据中删除指标
        del meta["metrics"][metric_name]
        self._save_json(self.meta_file, meta)
        
        # 遍历所有数据文件，删除该指标的数据
        for data_file in self.data_dir.glob("stats_*.json"):
            try:
                data = self._load_json(data_file)
                if metric_name in data:
                    del data[metric_name]
                    # 如果文件中还有其他数据，保存更新后的文件
                    if data:
                        self._save_json(data_file, data)
                    # 如果文件变空了，删除文件
                    else:
                        data_file.unlink()
            except Exception:
                # 忽略单个文件的错误，继续处理其他文件
                pass
                
        return True

    def delete_old_data(self, days_to_keep: int = 30):
        """删除旧数据"""
        cutoff_date = (datetime.now() - timedelta(days=days_to_keep)).date()

        # 遍历数据目录中的所有文件
        for data_file in self.data_dir.glob("stats_*.json"):
            try:
                # 从文件名中提取日期
                date_str = data_file.stem.replace("stats_", "")
                file_date = datetime.strptime(date_str, "%Y-%m-%d").date()

                # 如果文件日期早于截止日期，删除文件
                if file_date < cutoff_date:
                    data_file.unlink()
            except (ValueError, OSError):
                # 忽略无法解析或删除的文件
                continue



"""
统计数据可视化模块

提供终端图形化展示功能
"""

import os
import io
from typing import Dict, List, Optional, Any
from collections import OrderedDict
import plotext as plt
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich import box


class StatsVisualizer:
    """统计数据可视化类"""

    def __init__(self, width: Optional[int] = None, height: Optional[int] = None):
        """
        初始化可视化器

        Args:
            width: 图表宽度，默认为终端宽度-10
            height: 图表高度，默认为20
        """
        self.width = width or self._get_terminal_width() - 10
        self.height = height or 20

        # 确保最小尺寸
        self.width = max(self.width, 40)
        self.height = max(self.height, 10)

        # 初始化Rich Console
        self.console = Console()

    def _get_terminal_width(self) -> int:
        """获取终端宽度"""
        try:
            columns = os.get_terminal_size().columns
            return columns
        except:
            return 80

    def plot_line_chart(
        self,
        data: Dict[str, float],
        title: str = "",
        unit: Optional[str] = None,
        show_values: bool = True,
    ) -> str:
        """
        使用 plotext 绘制折线图
        """
        if not data:
            return "无数据可显示"

        sorted_data = OrderedDict(sorted(data.items()))
        labels = list(sorted_data.keys())
        values = list(sorted_data.values())

        plt.clf()
        plt.plotsize(self.width, self.height)
        plt.plot(values)
        plt.xticks(range(len(labels)), labels)
        if title:
            plt.title(title)
        if unit:
            plt.ylabel(unit)

        chart = plt.build()

        if show_values and values:
            min_val = min(values)
            max_val = max(values)
            avg_val = sum(values) / len(values)
            stats_info_text = (
                f"最小值: {min_val:.2f}, 最大值: {max_val:.2f}, 平均值: {avg_val:.2f}"
            )

            # 使用StringIO捕获Panel输出
            string_io = io.StringIO()
            temp_console = Console(file=string_io, width=self.width)
            temp_console.print(
                Panel(
                    stats_info_text,
                    title="[bold]数据统计[/bold]",
                    expand=False,
                    style="dim",
                    border_style="blue",
                )
            )
            stats_panel_str = string_io.getvalue()

            return chart + "\n" + stats_panel_str.strip()
        return chart

    def plot_bar_chart(
        self,
        data: Dict[str, float],
        title: str = "",
        unit: Optional[str] = None,
        horizontal: bool = False,
    ) -> str:
        """
        使用 plotext 绘制柱状图
        """
        if not data:
            return "无数据可显示"

        labels = list(data.keys())
        values = list(data.values())

        plt.clf()
        plt.plotsize(self.width, self.height)

        if horizontal:
            plt.bar(labels, values, orientation="horizontal")
        else:
            plt.bar(labels, values)
        if title:
            plt.title(title)
        if unit:
            plt.ylabel(unit)

        return plt.build()

    def show_summary(
        self,
        aggregated_data: Dict[str, Dict[str, Any]],
        metric_name: str,
        unit: Optional[str] = None,
        tags_filter: Optional[Dict[str, str]] = None,
    ) -> str:
        """
        显示数据摘要

        Args:
            aggregated_data: 聚合后的数据
            metric_name: 指标名称
            unit: 单位
            tags_filter: 标签过滤条件

        Returns:
            摘要字符串（用于兼容性，实际会直接打印）
        """
        if not aggregated_data:
            self.console.print("[yellow]无数据可显示[/yellow]")
            return "无数据可显示"

        # 创建表格
        table = Table(title=f"{metric_name} 统计摘要", box=box.ROUNDED)

        # 添加列
        table.add_column("时间", justify="center", style="cyan")
        table.add_column("计数", justify="right", style="green")
        table.add_column("总和", justify="right", style="yellow")
        table.add_column("平均", justify="right", style="yellow")
        table.add_column("最小", justify="right", style="blue")
        table.add_column("最大", justify="right", style="red")

        # 添加数据行
        for time_key, stats in sorted(aggregated_data.items()):
            table.add_row(
                time_key,
                str(stats["count"]),
                f"{stats['sum']:.2f}",
                f"{stats['avg']:.2f}",
                f"{stats['min']:.2f}",
                f"{stats['max']:.2f}",
            )

        # 显示表格
        self.console.print(table)

        # 显示单位信息
        if unit:
            self.console.print(f"\n[dim]单位: {unit}[/dim]")

        # 显示过滤条件
        if tags_filter:
            filter_str = ", ".join([f"{k}={v}" for k, v in tags_filter.items()])
            self.console.print(f"[dim]过滤条件: {filter_str}[/dim]")

        return ""  # 返回空字符串，实际输出已经通过console打印

    def show_table(
        self,
        records: List[Dict[str, Any]],
        metric_name: str,
        unit: Optional[str] = None,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        tags_filter: Optional[Dict[str, str]] = None,
    ) -> str:
        """
        使用Rich Table显示数据记录

        Args:
            records: 数据记录列表
            metric_name: 指标名称
            unit: 单位
            start_time: 开始时间
            end_time: 结束时间
            tags_filter: 标签过滤条件

        Returns:
            空字符串（实际通过console打印）
        """
        if not records:
            self.console.print(f"[yellow]没有找到指标 '{metric_name}' 的数据[/yellow]")
            return ""

        # 创建表格
        table = Table(title=f"指标: {metric_name}", box=box.ROUNDED)

        # 添加列
        table.add_column("时间", style="cyan", no_wrap=True)
        table.add_column("值", justify="right", style="yellow")
        table.add_column("标签", style="dim")

        # 只显示最近的20条记录
        display_records = records[-20:] if len(records) > 20 else records

        # 添加数据行
        from datetime import datetime

        for record in display_records:
            timestamp = datetime.fromisoformat(record["timestamp"])
            time_str = timestamp.strftime("%Y-%m-%d %H:%M:%S")
            value = f"{record['value']:.2f}"
            tags_str = ", ".join(f"{k}={v}" for k, v in record.get("tags", {}).items())

            table.add_row(time_str, value, tags_str)

        # 显示表格
        self.console.print(table)

        # 显示元信息
        info_items = []
        if unit:
            info_items.append(f"单位: {unit}")
        if start_time and end_time:
            info_items.append(f"时间范围: [cyan]{start_time}[/] ~ [cyan]{end_time}[/]")
        if tags_filter:
            filter_str = ", ".join([f"{k}={v}" for k, v in tags_filter.items()])
            info_items.append(f"过滤条件: {filter_str}")

        if info_items:
            self.console.print(
                Panel(
                    " | ".join(info_items),
                    title="[bold]查询详情[/bold]",
                    expand=False,
                    style="dim",
                    border_style="green",
                )
            )

        # 统计信息
        if len(records) > 0:
            values = [r["value"] for r in records]
            stats_info_text = (
                f"总记录数: {len(records)} | "
                f"显示: {len(display_records)} | "
                f"最小值: {min(values):.2f} | "
                f"最大值: {max(values):.2f} | "
                f"平均值: {sum(values)/len(values):.2f}"
            )
            self.console.print(
                Panel(
                    stats_info_text,
                    title="[bold]数据统计[/bold]",
                    expand=False,
                    style="dim",
                    border_style="blue",
                )
            )

        return ""



# -*- coding: utf-8 -*-
"""
输出格式化模块
该模块为Jarvis系统提供了丰富的文本格式化和显示工具。
包含：
- 用于分类不同输出类型的OutputType枚举
- 用于格式化和显示样式化输出的PrettyOutput类
- 多种编程语言的语法高亮支持
- 结构化输出的面板显示
"""
from datetime import datetime
from enum import Enum
from typing import Dict, Optional, Tuple, Any

from pygments.lexers import guess_lexer
from pygments.util import ClassNotFound
from rich.box import SIMPLE
from rich.panel import Panel
from rich.style import Style as RichStyle
from rich.syntax import Syntax
from rich.text import Text

from jarvis.jarvis_utils.config import get_pretty_output
from jarvis.jarvis_utils.globals import console, get_agent_list


class OutputType(Enum):
    """
    输出类型枚举，用于分类和样式化不同类型的消息。

    属性：
        SYSTEM: AI助手消息
        CODE: 代码相关输出
        RESULT: 工具执行结果
        ERROR: 错误信息
        INFO: 系统提示
        PLANNING: 任务规划
        PROGRESS: 执行进度
        SUCCESS: 成功信息
        WARNING: 警告信息
        DEBUG: 调试信息
        USER: 用户输入
        TOOL: 工具调用
    """

    SYSTEM = "SYSTEM"
    CODE = "CODE"
    RESULT = "RESULT"
    ERROR = "ERROR"
    INFO = "INFO"
    PLANNING = "PLANNING"
    PROGRESS = "PROGRESS"
    SUCCESS = "SUCCESS"
    WARNING = "WARNING"
    DEBUG = "DEBUG"
    USER = "USER"
    TOOL = "TOOL"


class PrettyOutput:
    """
    使用rich库格式化和显示富文本输出的类。

    提供以下方法：
    - 使用适当的样式格式化不同类型的输出
    - 代码块的语法高亮
    - 结构化内容的面板显示
    - 渐进显示的流式输出
    """

    # 不同输出类型的图标
    _ICONS = {
        OutputType.SYSTEM: "🤖",
        OutputType.CODE: "📝",
        OutputType.RESULT: "✨",
        OutputType.ERROR: "❌",
        OutputType.INFO: "ℹ️",
        OutputType.PLANNING: "📋",
        OutputType.PROGRESS: "⏳",
        OutputType.SUCCESS: "✅",
        OutputType.WARNING: "⚠️",
        OutputType.DEBUG: "🔍",
        OutputType.USER: "👤",
        OutputType.TOOL: "🔧",
    }
    # 语法高亮的语言映射
    _lang_map = {
        "Python": "python",
        "JavaScript": "javascript",
        "TypeScript": "typescript",
        "Java": "java",
        "C++": "cpp",
        "C#": "csharp",
        "Ruby": "ruby",
        "PHP": "php",
        "Go": "go",
        "Rust": "rust",
        "Bash": "bash",
        "HTML": "html",
        "CSS": "css",
        "SQL": "sql",
        "R": "r",
        "Kotlin": "kotlin",
        "Swift": "swift",
        "Scala": "scala",
        "Perl": "perl",
        "Lua": "lua",
        "YAML": "yaml",
        "JSON": "json",
        "XML": "xml",
        "Markdown": "markdown",
        "Text": "text",
        "Shell": "bash",
        "Dockerfile": "dockerfile",
        "Makefile": "makefile",
        "INI": "ini",
        "TOML": "toml",
    }

    @staticmethod
    def _detect_language(text: str, default_lang: str = "markdown") -> str:
        """
        检测给定文本的编程语言。

        参数：
            text: 要分析的文本
            default_lang: 如果检测失败，默认返回的语言

        返回：
            str: 检测到的语言名称
        """
        try:
            lexer = guess_lexer(text)
            detected_lang = lexer.name  # type: ignore[attr-defined]
            return PrettyOutput._lang_map.get(detected_lang, default_lang)
        except (ClassNotFound, Exception):
            return default_lang

    @staticmethod
    def _format(output_type: OutputType, timestamp: bool = True) -> str:
        """
        使用时间戳和图标格式化输出头。

        参数：
            output_type: 输出类型
            timestamp: 是否包含时间戳

        返回：
            Text: 格式化后的rich Text对象
        """
        icon = PrettyOutput._ICONS.get(output_type, "")
        formatted = f"{icon}  "
        if timestamp:
            formatted += f"[{datetime.now().strftime('%H:%M:%S')}][{output_type.value}]"
        agent_info = get_agent_list()
        if agent_info:
            formatted += f"[{agent_info}]"
        return formatted

    @staticmethod
    def print(
        text: str,
        output_type: OutputType,
        timestamp: bool = True,
        lang: Optional[str] = None,
        traceback: bool = False,
    ):
        """
        使用样式和语法高亮打印格式化输出。

        参数：
            text: 要打印的文本内容
            output_type: 输出类型（影响样式）
            timestamp: 是否显示时间戳
            lang: 语法高亮的语言
            traceback: 是否显示错误的回溯信息
        """
        styles: Dict[OutputType, Dict[str, Any]] = {
            OutputType.SYSTEM: dict(bgcolor="#1e2b3c"),
            OutputType.CODE: dict(bgcolor="#1c2b1c"),
            OutputType.RESULT: dict(bgcolor="#1c1c2b"),
            OutputType.ERROR: dict(bgcolor="#2b1c1c"),
            OutputType.INFO: dict(bgcolor="#2b2b1c", meta={"icon": "ℹ️"}),
            OutputType.PLANNING: dict(bgcolor="#2b1c2b"),
            OutputType.PROGRESS: dict(bgcolor="#1c1c1c"),
            OutputType.SUCCESS: dict(bgcolor="#1c2b1c"),
            OutputType.WARNING: dict(bgcolor="#2b2b1c"),
            OutputType.DEBUG: dict(bgcolor="#1c1c1c"),
            OutputType.USER: dict(bgcolor="#1c2b2b"),
            OutputType.TOOL: dict(bgcolor="#1c2b2b"),
        }

        header_styles = {
            OutputType.SYSTEM: RichStyle(
                color="bright_cyan", bgcolor="#1e2b3c", frame=True, meta={"icon": "🤖"}
            ),
            OutputType.CODE: RichStyle(
                color="green", bgcolor="#1c2b1c", frame=True, meta={"icon": "📝"}
            ),
            OutputType.RESULT: RichStyle(
                color="bright_blue", bgcolor="#1c1c2b", frame=True, meta={"icon": "✨"}
            ),
            OutputType.ERROR: RichStyle(
                color="red", frame=True, bgcolor="#2b1c1c", meta={"icon": "❌"}
            ),
            OutputType.INFO: RichStyle(
                color="bright_cyan", frame=True, bgcolor="#2b2b1c", meta={"icon": "ℹ️"}
            ),
            OutputType.PLANNING: RichStyle(
                color="purple",
                bold=True,
                frame=True,
                bgcolor="#2b1c2b",
                meta={"icon": "📋"},
            ),
            OutputType.PROGRESS: RichStyle(
                color="white",
                encircle=True,
                frame=True,
                bgcolor="#1c1c1c",
                meta={"icon": "⏳"},
            ),
            OutputType.SUCCESS: RichStyle(
                color="bright_green",
                bold=True,
                strike=False,
                bgcolor="#1c2b1c",
                meta={"icon": "✅"},
            ),
            OutputType.WARNING: RichStyle(
                color="yellow",
                bold=True,
                blink2=True,
                bgcolor="#2b2b1c",
                meta={"icon": "⚠️"},
            ),
            OutputType.DEBUG: RichStyle(
                color="grey58",
                dim=True,
                conceal=True,
                bgcolor="#1c1c1c",
                meta={"icon": "🔍"},
            ),
            OutputType.USER: RichStyle(
                color="spring_green2",
                frame=True,
                bgcolor="#1c2b2b",
                meta={"icon": "👤"},
            ),
            OutputType.TOOL: RichStyle(
                color="dark_sea_green4",
                bgcolor="#1c2b2b",
                frame=True,
                meta={"icon": "🔧"},
            ),
        }

        lang = (
            lang
            if lang is not None
            else PrettyOutput._detect_language(text, default_lang="markdown")
        )
        header = Text(
            PrettyOutput._format(output_type, timestamp),
            style=header_styles[output_type],
        )
        content = Syntax(
            text,
            lang,
            theme="monokai",
            word_wrap=True,
            background_color=styles[output_type]["bgcolor"],
        )
        panel = Panel(
            content,
            border_style=header_styles[output_type],
            title=header,
            title_align="left",
            padding=(0, 0),
            highlight=True,
        )
        if get_pretty_output():
            console.print(panel)
        else:
            if len(text.strip().splitlines()) > 1:
                console.print(header)
                console.print(content)
            else:
                console.print(header, content)
        if traceback or output_type == OutputType.ERROR:
            try:
                console.print_exception()
            except Exception as e:
                console.print(f"Error: {e}")

    @staticmethod
    def section(title: str, output_type: OutputType = OutputType.INFO):
        """
        在样式化面板中打印章节标题。

        参数：
            title: 章节标题文本
            output_type: 输出类型（影响样式）
        """
        text = Text(title, style=output_type.value, justify="center")
        panel = Panel(text, border_style=output_type.value)
        if get_pretty_output():
            console.print(panel)
        else:
            console.print(text)

    @staticmethod
    def print_gradient_text(
        text: str, start_color: Tuple[int, int, int], end_color: Tuple[int, int, int]
    ) -> None:
        """打印带有渐变色彩的文本。

        Args:
            text: 要打印的文本
            start_color: 起始RGB颜色元组 (r, g, b)
            end_color: 结束RGB颜色元组 (r, g, b)
        """
        lines = text.strip("\n").split("\n")
        total_lines = len(lines)
        colored_lines = []
        for i, line in enumerate(lines):
            # 计算当前行的渐变颜色
            r = int(
                start_color[0] + (end_color[0] - start_color[0]) * i / (total_lines - 1)
            )
            g = int(
                start_color[1] + (end_color[1] - start_color[1]) * i / (total_lines - 1)
            )
            b = int(
                start_color[2] + (end_color[2] - start_color[2]) * i / (total_lines - 1)
            )

            # 使用ANSI转义序列设置颜色
            colored_lines.append(f"\033[38;2;{r};{g};{b}m{line}\033[0m")
        colored_text = Text(
            "\n".join(colored_lines), style=OutputType.TOOL.value, justify="center"
        )
        panel = Panel(colored_text, box=SIMPLE)
        console.print(panel)



# -*- coding: utf-8 -*-
# Kimi 平台实现模块
# 提供与 Moonshot AI 的 Kimi 大模型交互功能
import json
import mimetypes
import os
import time
from typing import Dict, Generator, List, Tuple

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils import http
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.utils import while_success


class KimiModel(BasePlatform):
    """Kimi 大模型平台实现类
    封装了与 Kimi 大模型交互的所有功能，包括：
    - 会话管理
    - 文件上传
    - 消息收发
    """

    def get_model_list(self) -> List[Tuple[str, str]]:
        """Get model list"""
        return [
            ("k1.5", "基于网页的 Kimi，免费接口"),
            ("k1.5-thinking", "基于网页的 Kimi，深度思考模型"),
            ("k2", "基于网页的 Kimi，深度思考模型 K2"),
        ]

    def __init__(self):
        """
        Initialize Kimi model
        """
        super().__init__()
        self.chat_id = ""  # 当前会话ID
        self.api_key = os.getenv("KIMI_API_KEY")  # 从环境变量获取API密钥
        if not self.api_key:
            PrettyOutput.print("KIMI_API_KEY 未设置", OutputType.WARNING)
        self.auth_header = f"Bearer {self.api_key}"  # 认证头信息
        self.uploaded_files = []  # 存储已上传文件的信息
        self.chat_id = ""  # 当前会话ID
        self.first_chat = True  # 标记是否是第一次对话
        self.system_message = ""  # 系统提示消息
        self.model_name = "kimi"  # 默认模型名称

    def set_system_prompt(self, message: str):
        """Set system message"""
        self.system_message = message

    def set_model_name(self, model_name: str):
        """Set model name"""
        self.model_name = model_name

    def _create_chat(self) -> bool:
        """Create a new chat session"""
        url = "https://kimi.moonshot.cn/api/chat"
        payload = json.dumps(
            {"name": "Unnamed session", "is_example": False, "kimiplus_id": "kimi"}
        )
        headers = {
            "Authorization": self.auth_header,
            "Content-Type": "application/json",
        }
        try:
            response = while_success(
                lambda: http.post(url, headers=headers, data=payload),
                sleep_time=5,
            )
            if response.status_code != 200:
                PrettyOutput.print(
                    f"错误：创建会话失败：{response.json()}", OutputType.ERROR
                )
                return False
            self.chat_id = response.json()["id"]
            return True
        except Exception as e:
            PrettyOutput.print(f"错误：创建会话失败：{e}", OutputType.ERROR)
            return False

    def _get_presigned_url(self, filename: str, action: str) -> Dict:
        """Get presigned upload URL"""
        url = "https://kimi.moonshot.cn/api/pre-sign-url"

        payload = json.dumps(
            {"action": action, "name": os.path.basename(filename)}, ensure_ascii=False
        )

        headers = {
            "Authorization": self.auth_header,
            "Content-Type": "application/json",
        }

        response = while_success(
            lambda: http.post(url, headers=headers, data=payload), sleep_time=5
        )
        return response.json()

    def support_upload_files(self) -> bool:
        """Check if platform supports upload files"""
        return True

    def _upload_file(self, file_path: str, presigned_url: str) -> bool:
        """Upload file to presigned URL"""
        try:
            with open(file_path, "rb") as f:
                content = f.read()
                response = while_success(
                    lambda: http.put(presigned_url, data=content), sleep_time=5
                )
                return response.status_code == 200
        except Exception as e:
            PrettyOutput.print(f"错误：上传文件失败：{e}", OutputType.ERROR)
            return False

    def _get_file_info(self, file_data: Dict, name: str, file_type: str) -> Dict:
        """Get file information"""
        url = "https://kimi.moonshot.cn/api/file"
        payload = json.dumps(
            {
                "type": file_type,
                "name": name,
                "object_name": file_data["object_name"],
                "chat_id": self.chat_id,
                "file_id": file_data.get("file_id", ""),
            },
            ensure_ascii=False,
        )

        headers = {
            "Authorization": self.auth_header,
            "Content-Type": "application/json",
        }

        response = while_success(
            lambda: http.post(url, headers=headers, data=payload), sleep_time=5
        )
        return response.json()

    def _wait_for_parse(self, file_id: str) -> bool:
        """Wait for file parsing to complete"""
        url = "https://kimi.moonshot.cn/api/file/parse_process"
        headers = {
            "Authorization": self.auth_header,
            "Content-Type": "application/json",
        }

        max_retries = 30
        retry_count = 0

        while retry_count < max_retries:
            payload = {"ids": [file_id]}
            response_stream = while_success(
                lambda: http.stream_post(url, headers=headers, json=payload),
                sleep_time=5,
            )

            # 处理流式响应
            for line in response_stream:
                if not line.strip():
                    continue

                # SSE格式的行通常以"data: "开头
                if line.startswith("data: "):
                    try:
                        data = json.loads(line[6:])
                        if data.get("event") == "resp":
                            status = data.get("file_info", {}).get("status")
                            if status == "parsed":
                                return True
                            elif status == "failed":
                                return False
                    except json.JSONDecodeError:
                        continue

            retry_count += 1
            time.sleep(1)

        return False

    def upload_files(self, file_list: List[str]) -> bool:
        """Upload file list and return file information"""
        if not file_list:
            return True

        if not self.chat_id:
            print("🚀 正在创建聊天会话...")
            if not self._create_chat():
                print("❌ 创建聊天会话失败")
                return False
            print("✅ 创建聊天会话成功")

        uploaded_files = []
        for index, file_path in enumerate(file_list, 1):
            file_name = os.path.basename(file_path)
            print(f"🔍 处理文件 [{index}/{len(file_list)}]: {file_name}")
            try:
                mime_type, _ = mimetypes.guess_type(file_path)
                action = (
                    "image" if mime_type and mime_type.startswith("image/") else "file"
                )

                # 获取预签名URL
                print(f"🔍 获取上传URL: {file_name}")
                presigned_data = self._get_presigned_url(file_path, action)

                # 上传文件
                print(f"🔍 上传文件: {file_name}")
                if self._upload_file(file_path, presigned_data["url"]):
                    # 获取文件信息
                    print(f"🔍 获取文件信息: {file_name}")
                    file_info = self._get_file_info(presigned_data, file_name, action)

                    # 只有文件需要解析
                    if action == "file":
                        print(f"🔍 等待文件解析: {file_name}")
                        if self._wait_for_parse(file_info["id"]):
                            uploaded_files.append(file_info)
                            print(f"✅ 文件处理完成: {file_name}")
                        else:
                            print(f"❌ 文件解析失败: {file_name}")
                            return False
                    else:
                        uploaded_files.append(file_info)
                        print(f"✅ 图片处理完成: {file_name}")
                else:
                    print(f"❌ 文件上传失败: {file_name}")
                    return False

            except Exception as e:
                print(f"❌ 处理文件出错 {file_path}: {str(e)}")
                return False

        self.uploaded_files = uploaded_files
        return True

    def chat(self, message: str) -> Generator[str, None, None]:
        """发送消息并获取响应流
        参数:
            message: 要发送的消息内容
        返回:
            生成器，逐块返回模型响应
        """
        if not self.chat_id:
            if not self._create_chat():
                raise Exception("Failed to create chat session")

        url = f"https://kimi.moonshot.cn/api/chat/{self.chat_id}/completion/stream"

        refs = []
        refs_file = []
        if self.uploaded_files:
            refs = [f["id"] for f in self.uploaded_files]
            refs_file = self.uploaded_files
            self.uploaded_files = []

        if self.first_chat:
            message = self.system_message + "\n" + message
            self.first_chat = False

        payload = {
            "messages": [{"role": "user", "content": message}],
            "use_search": True if self.web else False,
            "extend": {"sidebar": True},
            "kimiplus_id": "kimi",
            "use_deep_research": False,
            "use_semantic_memory": True,
            "history": [],
            "refs": refs,
            "refs_file": refs_file,
            "model": self.model_name,
        }

        headers = {
            "Authorization": self.auth_header,
            "Content-Type": "application/json",
        }

        try:
            # 使用新的stream_post接口发送消息请求，获取流式响应
            response_stream = while_success(
                lambda: http.stream_post(url, headers=headers, json=payload),
                sleep_time=5,
            )

            # 处理流式响应
            for line in response_stream:
                if not line.strip():
                    continue

                # SSE格式的行通常以"data: "开头
                if line.startswith("data: "):
                    try:
                        data = json.loads(line[6:])
                        event = data.get("event")

                        if event == "cmpl":
                            # 处理补全文本
                            text = data.get("text", "")
                            if text:
                                yield text
                    except json.JSONDecodeError:
                        continue

            return None

        except Exception as e:
            raise Exception(f"Chat failed: {str(e)}")

    def delete_chat(self) -> bool:
        """Delete current session"""
        if not self.chat_id:
            return True  # 如果没有会话ID，视为删除成功

        url = f"https://kimi.moonshot.cn/api/chat/{self.chat_id}"
        headers = {
            "Authorization": self.auth_header,
            "Content-Type": "application/json",
        }

        try:
            response = while_success(
                lambda: http.delete(url, headers=headers), sleep_time=5
            )
            if response.status_code == 200:
                self.chat_id = ""
                self.uploaded_files = []
                self.first_chat = True  # 重置first_chat标记
                return True
            else:
                PrettyOutput.print(
                    f"删除会话失败: HTTP {response.status_code}", OutputType.WARNING
                )
                return False
        except Exception as e:
            PrettyOutput.print(f"删除会话时发生错误: {str(e)}", OutputType.ERROR)
            return False

    def save(self, file_path: str) -> bool:
        """Save chat session to a file."""
        if not self.chat_id:
            PrettyOutput.print("没有活动的会话可供保存", OutputType.WARNING)
            return False

        state = {
            "chat_id": self.chat_id,
            "model_name": self.model_name,
            "system_message": self.system_message,
            "first_chat": self.first_chat,
            "uploaded_files": self.uploaded_files,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.print(f"会话已成功保存到 {file_path}", OutputType.SUCCESS)
            return True
        except Exception as e:
            PrettyOutput.print(f"保存会话失败: {str(e)}", OutputType.ERROR)
            return False

    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.chat_id = state.get("chat_id", "")
            self.model_name = state.get("model_name", "kimi")
            self.system_message = state.get("system_message", "")
            self.first_chat = state.get("first_chat", True)
            self.uploaded_files = state.get("uploaded_files", [])
            self._saved = True

            PrettyOutput.print(f"从 {file_path} 成功恢复会话", OutputType.SUCCESS)
            return True
        except FileNotFoundError:
            PrettyOutput.print(f"会话文件未找到: {file_path}", OutputType.ERROR)
            return False
        except Exception as e:
            PrettyOutput.print(f"恢复会话失败: {str(e)}", OutputType.ERROR)
            return False

    def name(self) -> str:
        """Model name"""
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """Platform name"""
        return "kimi"

    def support_web(self) -> bool:
        """Kimi平台支持web功能"""
        return True

    @classmethod
    def get_required_env_keys(cls) -> List[str]:
        """
        获取Kimi平台所需的环境变量键列表

        返回:
            List[str]: 环境变量键的列表
        """
        return ["KIMI_API_KEY"]



# -*- coding: utf-8 -*-
"""
Utility module for loading language-specific code review checklists.
"""
from typing import Dict, Optional

# Import checklist modules
from jarvis.jarvis_code_analysis.checklists import (
    c_cpp,
    csharp,
    data_format,
    devops,
    docs,
    go,
    infrastructure,
    java,
    javascript,
    kotlin,
    php,
    python,
    ruby,
    rust,
    shell,
    sql,
    swift,
    web,
)

# Map of language identifiers to their checklist content
CHECKLIST_MAP = {
    "c_cpp": c_cpp.CHECKLIST,
    "go": go.CHECKLIST,
    "python": python.CHECKLIST,
    "rust": rust.CHECKLIST,
    "java": java.CHECKLIST,
    "javascript": javascript.CHECKLIST,
    "typescript": javascript.CHECKLIST,  # TypeScript uses the same checklist as JavaScript
    "csharp": csharp.CHECKLIST,
    "swift": swift.CHECKLIST,
    "php": php.CHECKLIST,
    "shell": shell.CHECKLIST,
    "sql": sql.CHECKLIST,
    "ruby": ruby.CHECKLIST,
    "kotlin": kotlin.CHECKLIST,
    "html": web.CHECKLIST,
    "css": web.CHECKLIST,
    "xml": data_format.CHECKLIST,
    "json": data_format.CHECKLIST,
    "yaml": data_format.CHECKLIST,
    "docker": infrastructure.CHECKLIST,
    "terraform": infrastructure.CHECKLIST,
    "markdown": docs.CHECKLIST,
    "docs": docs.CHECKLIST,
    "makefile": devops.CHECKLIST,
    "devops": devops.CHECKLIST,
}


def get_language_checklist(language: str) -> Optional[str]:
    """
    Get the checklist for a specific language.

    Args:
        language: The language identifier ('c_cpp', 'go', 'python', 'rust', etc.)

    Returns:
        The checklist content as a string, or None if not found
    """
    return CHECKLIST_MAP.get(language)


def get_all_checklists() -> Dict[str, str]:
    """
    Get all available language checklists.

    Returns:
        Dictionary mapping language identifiers to their checklist content
    """
    return CHECKLIST_MAP



# -*- coding: utf-8 -*-
from typing import Any, Tuple

from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.input import user_confirm


def shell_input_handler(user_input: str, agent: Any) -> Tuple[str, bool]:
    lines = user_input.splitlines()
    cmdline = [line for line in lines if line.startswith("!")]
    if len(cmdline) == 0:
        return user_input, False
    else:
        script = "\n".join([c[1:] for c in cmdline])
        PrettyOutput.print(script, OutputType.CODE, lang="bash")
        if user_confirm(f"是否要执行以上shell脚本？", default=True):
            from jarvis.jarvis_tools.registry import ToolRegistry

            output = ToolRegistry().handle_tool_calls(
                {
                    "name": "execute_script",
                    "want": "提取命令执行结果关键信息",
                    "arguments": {"interpreter": "bash", "script_content": script},
                },
                agent,
            )
            if user_confirm("是否将执行结果反馈给Agent？", default=True):
                return (
                    f"{user_input}\n\n用户执行以下脚本：\n{script}\n\n执行结果：\n{output}",
                    False,
                )
            return "", True
        return user_input, False



# -*- coding: utf-8 -*-
import json
import time
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, List

from jarvis.jarvis_utils.config import get_data_dir
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.globals import add_short_term_memory


class SaveMemoryTool:
    """保存记忆工具，用于将信息保存到长短期记忆系统"""

    name = "save_memory"
    description = """保存信息到长短期记忆系统。
    
    支持批量保存多条记忆，可以同时保存不同类型的记忆。
    
    支持的记忆类型：
    - project_long_term: 项目长期记忆（与当前项目相关的信息）
    - global_long_term: 全局长期记忆（通用的信息、用户喜好、知识、方法等）
    - short_term: 短期记忆（当前任务相关的信息）
    
    项目长期记忆存储在当前目录的 .jarvis/memory 下
    全局长期记忆和短期记忆存储在数据目录的 memory 子目录下
    """

    parameters = {
        "type": "object",
        "properties": {
            "memories": {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "memory_type": {
                            "type": "string",
                            "enum": [
                                "project_long_term",
                                "global_long_term",
                                "short_term",
                            ],
                            "description": "记忆类型",
                        },
                        "tags": {
                            "type": "array",
                            "items": {"type": "string"},
                            "description": "用于索引记忆的标签列表",
                        },
                        "content": {
                            "type": "string",
                            "description": "要保存的记忆内容",
                        },
                    },
                    "required": ["memory_type", "tags", "content"],
                },
                "description": "要保存的记忆列表",
            }
        },
        "required": ["memories"],
    }

    def __init__(self):
        """初始化保存记忆工具"""
        self.project_memory_dir = Path(".jarvis/memory")
        self.global_memory_dir = Path(get_data_dir()) / "memory"

    def _get_memory_dir(self, memory_type: str) -> Path:
        """根据记忆类型获取存储目录"""
        if memory_type == "project_long_term":
            return self.project_memory_dir
        elif memory_type in ["global_long_term", "short_term"]:
            return self.global_memory_dir / memory_type
        else:
            raise ValueError(f"未知的记忆类型: {memory_type}")

    def _generate_memory_id(self) -> str:
        """生成唯一的记忆ID"""
        # 添加微秒级时间戳确保唯一性
        time.sleep(0.001)  # 确保不同记忆有不同的时间戳
        return datetime.now().strftime("%Y%m%d_%H%M%S_%f")

    def _save_single_memory(self, memory_data: Dict[str, Any]) -> Dict[str, Any]:
        """保存单条记忆"""
        memory_type = memory_data["memory_type"]
        tags = memory_data.get("tags", [])
        content = memory_data.get("content", "")

        # 生成记忆ID
        memory_id = self._generate_memory_id()

        # 创建记忆对象
        memory_obj = {
            "id": memory_id,
            "type": memory_type,
            "tags": tags,
            "content": content,
            "created_at": datetime.now().isoformat(),
            "updated_at": datetime.now().isoformat(),
        }

        if memory_type == "short_term":
            # 短期记忆保存到全局变量
            add_short_term_memory(memory_obj)

            result = {
                "memory_id": memory_id,
                "memory_type": memory_type,
                "tags": tags,
                "storage": "memory",
                "message": f"短期记忆已成功保存到内存，ID: {memory_id}",
            }
        else:
            # 长期记忆保存到文件
            # 获取存储目录并确保存在
            memory_dir = self._get_memory_dir(memory_type)
            memory_dir.mkdir(parents=True, exist_ok=True)

            # 保存记忆文件
            memory_file = memory_dir / f"{memory_id}.json"
            with open(memory_file, "w", encoding="utf-8") as f:
                json.dump(memory_obj, f, ensure_ascii=False, indent=2)

            result = {
                "memory_id": memory_id,
                "memory_type": memory_type,
                "tags": tags,
                "file_path": str(memory_file),
                "message": f"记忆已成功保存，ID: {memory_id}",
            }

        return result

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """执行保存记忆操作"""
        try:
            memories = args.get("memories", [])

            if not memories:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": "没有提供要保存的记忆",
                }

            results = []
            success_count = 0
            failed_count = 0

            # 保存每条记忆
            for i, memory_data in enumerate(memories):
                try:
                    result = self._save_single_memory(memory_data)
                    results.append(result)
                    success_count += 1

                    # 打印单条记忆保存信息
                    memory_type = memory_data["memory_type"]
                    tags = memory_data.get("tags", [])
                    PrettyOutput.print(
                        f"[{i+1}/{len(memories)}] {memory_type} 记忆已保存\n"
                        f"ID: {result['memory_id']}\n"
                        f"标签: {', '.join(tags)}",
                        OutputType.SUCCESS,
                    )
                except Exception as e:
                    failed_count += 1
                    error_msg = f"保存第 {i+1} 条记忆失败: {str(e)}"
                    PrettyOutput.print(error_msg, OutputType.ERROR)
                    results.append(
                        {
                            "error": error_msg,
                            "memory_type": memory_data.get("memory_type", "unknown"),
                            "tags": memory_data.get("tags", []),
                        }
                    )

            # 生成总结报告
            PrettyOutput.print(
                f"\n批量保存完成：成功 {success_count} 条，失败 {failed_count} 条",
                OutputType.INFO,
            )

            # 构建返回结果
            output = {
                "total": len(memories),
                "success": success_count,
                "failed": failed_count,
                "results": results,
            }

            return {
                "success": True,
                "stdout": json.dumps(output, ensure_ascii=False, indent=2),
                "stderr": "",
            }

        except Exception as e:
            error_msg = f"保存记忆失败: {str(e)}"
            PrettyOutput.print(error_msg, OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": error_msg}



# -*- coding: utf-8 -*-
"""
Infrastructure as Code (Docker, Terraform) specific code review checklist.
"""

CHECKLIST = """
## 基础设施即代码 (Docker, Terraform) 代码审查检查清单

### Docker 通用检查
- [ ] Dockerfile 是否遵循最佳实践
- [ ] 是否使用了适当的基础镜像
- [ ] 是否最小化了镜像层数
- [ ] 是否按照依赖稳定性顺序组织指令
- [ ] 是否移除了不必要的依赖和文件
- [ ] 是否避免安装调试工具和开发依赖
- [ ] 镜像标签是否明确 (避免使用 `latest`)
- [ ] 是否使用了多阶段构建减小镜像大小
- [ ] 是否设置了适当的工作目录 (WORKDIR)

### Docker 安全检查
- [ ] 是否以非 root 用户运行容器
- [ ] 是否限制了容器资源 (CPU, 内存)
- [ ] 是否避免了将敏感信息硬编码到镜像中
- [ ] 是否使用了安全扫描工具检查漏洞
- [ ] 是否设置了适当的文件和目录权限
- [ ] 是否避免了使用特权模式
- [ ] 是否移除了不必要的包和工具
- [ ] 是否在构建时验证下载的软件包

### Docker 优化检查
- [ ] 是否使用了 .dockerignore 文件
- [ ] 是否合并了相关命令减少层数
- [ ] 是否使用了缓存优化构建时间
- [ ] 是否优化了镜像大小
- [ ] 是否避免了不必要的环境变量
- [ ] 是否使用了合适的指令进行健康检查
- [ ] 是否为应用设置了适当的启动命令
- [ ] 构建参数和环境变量是否使用恰当

### Docker Compose 检查
- [ ] 服务配置是否合理
- [ ] 网络配置是否安全且高效
- [ ] 卷挂载是否正确
- [ ] 环境变量是否适当管理
- [ ] 依赖关系是否明确定义
- [ ] 资源限制是否设置
- [ ] 是否使用了版本控制
- [ ] 是否有适当的重启策略

### Terraform 通用检查
- [ ] 代码结构是否模块化和可重用
- [ ] 是否遵循 Terraform 目录结构最佳实践
- [ ] 变量和输出是否有明确的说明文档
- [ ] 资源名称是否一致且有意义
- [ ] 是否避免了硬编码值 (使用变量)
- [ ] 是否明确指定了 provider 版本
- [ ] 是否使用了远程状态存储和锁定
- [ ] 是否遵循命名约定

### Terraform 安全检查
- [ ] 敏感信息是否使用 Terraform 加密存储
- [ ] 是否限制了资源的权限和访问范围
- [ ] 网络安全组/防火墙规则是否遵循最小特权原则
- [ ] 是否启用了资源日志和监控
- [ ] 访问密钥和证书是否安全管理
- [ ] 是否使用循环依赖及其影响
- [ ] 是否正确使用了 IAM 策略和角色
- [ ] 是否考虑了数据加密（传输中和静态）

### Terraform 性能和优化
- [ ] 资源依赖关系是否明确定义
- [ ] 是否避免了创建不必要的资源
- [ ] 是否使用了条件表达式优化资源创建
- [ ] 是否使用了本地值减少重复
- [ ] 是否合理使用了数据源
- [ ] 是否使用 count 或 for_each 简化资源创建
- [ ] 是否优化了资源的创建顺序
- [ ] 是否考虑了高可用性和灾难恢复

### 基础设施即代码 (IaC) 最佳实践
- [ ] 是否实现了基础设施版本控制
- [ ] 是否有适当的变更管理流程
- [ ] 代码是否通过自动化测试
- [ ] 是否创建了可重复的环境
- [ ] 是否使用了不可变基础设施原则
- [ ] 是否考虑了扩展性需求
- [ ] 是否有足够的文档
- [ ] 是否考虑了成本优化

### 云服务提供商特定检查
- [ ] 是否遵循了云服务提供商的最佳实践
- [ ] 资源命名是否符合组织规范
- [ ] 是否设置了适当的资源标签
- [ ] 是否考虑了区域/可用区策略
- [ ] 是否有适当的备份和恢复策略
- [ ] 是否优化了资源成本
- [ ] 是否考虑了服务限制和配额
- [ ] 是否正确配置了自动扩展
"""



from typing import List
from .llm_interface import LLMInterface
from jarvis.jarvis_utils.output import PrettyOutput, OutputType


class QueryRewriter:
    """
    使用LLM将用户的查询重写为多个不同的搜索查询，以提高检索召回率。
    """

    def __init__(self, llm: LLMInterface):
        """
        初始化QueryRewriter。

        参数:
            llm: 实现LLMInterface接口的类的实例。
        """
        self.llm = llm
        self.rewrite_prompt_template = self._create_prompt_template()

    def _create_prompt_template(self) -> str:
        """为多查询重写任务创建提示模板。"""
        return """
你是一个精通检索和语言的AI助手。你的任务是将以下这个单一的用户问题，改写为几个语义相关但表达方式不同的搜索查询，并提供英文翻译。这有助于在多语言知识库中进行更全面的搜索。

请遵循以下原则：
1.  **保留核心意图**: 所有查询都必须围绕原始问题的核心意图。
2.  **查询类型**:
    - **同义词/相关术语查询**: 使用原始语言，通过替换同义词或相关术语来生成1-2个新的查询。
    - **英文翻译查询**: 将原始问题翻译成一个简洁的英文搜索查询。
3.  **简洁性**: 每个查询都应该是独立的、可以直接用于搜索的短语或问题。
4.  **严格格式要求**: 你必须将所有重写后的查询放置在 `<REWRITE>` 和 `</REWRITE>` 标签之间。每个查询占一行。不要在标签内外添加任何编号、前缀或解释。

示例输出格式:
<REWRITE>
使用不同表述的中文查询
另一个中文查询
English version of the query
</REWRITE>

原始问题:
---
{query}
---

请将改写后的查询包裹在 `<REWRITE>` 标签内:
"""

    def rewrite(self, query: str) -> List[str]:
        """
        使用LLM将用户查询重写为多个查询。

        参数:
            query: 原始用户查询。

        返回:
            一个经过重写、搜索优化的查询列表。
        """
        prompt = self.rewrite_prompt_template.format(query=query)
        PrettyOutput.print(
            "正在将原始查询重写为多个搜索查询...", output_type=OutputType.INFO, timestamp=False
        )

        import re

        max_retries = 3
        attempts = 0
        rewritten_queries = []
        response_text = ""

        while attempts < max_retries:
            attempts += 1
            response_text = self.llm.generate(prompt)
            match = re.search(r"<REWRITE>(.*?)</REWRITE>", response_text, re.DOTALL)

            if match:
                content = match.group(1).strip()
                rewritten_queries = [
                    line.strip() for line in content.split("\n") if line.strip()
                ]
                PrettyOutput.print(
                    f"成功从LLM响应中提取到内容 (尝试 {attempts}/{max_retries})。",
                    output_type=OutputType.SUCCESS,
                    timestamp=False,
                )
                break  # 提取成功，退出循环
            else:
                PrettyOutput.print(
                    f"未能从LLM响应中提取内容。正在重试... ({attempts}/{max_retries})",
                    output_type=OutputType.WARNING,
                    timestamp=False,
                )

        # 如果所有重试都失败，则跳过重写步骤
        if not rewritten_queries:
            PrettyOutput.print(
                "所有重试均失败。跳过查询重写，将仅使用原始查询。",
                output_type=OutputType.ERROR,
                timestamp=False,
            )

        # 同时包含原始查询以保证鲁棒性
        if query not in rewritten_queries:
            rewritten_queries.insert(0, query)

        PrettyOutput.print(
            f"生成了 {len(rewritten_queries)} 个查询变体。",
            output_type=OutputType.SUCCESS,
            timestamp=False,
        )
        return rewritten_queries



# -*- coding: utf-8 -*-
import json
import os
import re
import sys
import tempfile
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional, Protocol, Tuple

import yaml

from jarvis.jarvis_mcp import McpClient
from jarvis.jarvis_mcp.sse_mcp_client import SSEMcpClient
from jarvis.jarvis_mcp.stdio_mcp_client import StdioMcpClient
from jarvis.jarvis_mcp.streamable_mcp_client import StreamableMcpClient
from jarvis.jarvis_tools.base import Tool
from jarvis.jarvis_utils.config import get_data_dir, get_tool_load_dirs
from jarvis.jarvis_utils.input import user_confirm
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot
from jarvis.jarvis_utils.utils import is_context_overflow, daily_check_git_updates

tool_call_help = f"""
<tool_system_guide>
<introduction>
# 🛠️ 工具使用系统
您正在使用一个需要精确格式和严格规则的工具执行系统。
</introduction>

<format>
# 📋 工具调用格式
{ot("TOOL_CALL")}
want: 想要从执行结果中获取到的信息，如果工具输出内容过长，会根据此字段尝试提取有效信息
name: 工具名称
arguments:
  param1: 值1
  param2: 值2
{ct("TOOL_CALL")}
</format>

<rules>
# ❗ 关键规则
<rule>
### 1. 每次只使用一个工具
- 一次只执行一个工具
- 等待结果后再进行下一步
</rule>

<rule>
### 2. 严格遵守格式
- 完全按照上述格式
- 使用正确的YAML格式，2个空格作为缩进
- 包含所有必需参数
</rule>

<rule>
### 3. 结果处理
- 等待执行结果
- 不要假设结果
- 不要创建虚假响应
- 不要想象对话
</rule>

<rule>
### 4. 信息管理
- 如果信息不足，询问用户
- 跳过不必要的步骤
- 如果卡住，请求指导
- 不要在没有完整信息的情况下继续
</rule>
</rules>

<string_format>
# 📝 字符串参数格式
始终使用 |2 语法表示字符串参数，防止多行字符串行首空格引起歧义：

{ot("TOOL_CALL")}
want: 当前的git状态，期望获取xxx的提交记录
name: execute_script
arguments:
  interpreter: bash
  script_cotent: |2
    git status --porcelain
{ct("TOOL_CALL")}
</string_format>

<best_practices>
# 💡 最佳实践
- 准备好后立即开始执行
- 无需请求许可即可开始
- 使用正确的字符串格式
- 监控进度并调整
- 遇到困难时请求帮助
</best_practices>

<common_errors>
# ⚠️ 常见错误
- 同时调用多个工具
- 字符串参数缺少 |2
- 假设工具结果
- 创建虚构对话
- 在没有所需信息的情况下继续
- yaml 格式错误
</common_errors>
</tool_system_guide>
"""


class OutputHandlerProtocol(Protocol):
    def name(self) -> str: ...

    def can_handle(self, response: str) -> bool: ...

    def prompt(self) -> str: ...

    def handle(self, response: str, agent: Any) -> Tuple[bool, Any]: ...


class ToolRegistry(OutputHandlerProtocol):
    def name(self) -> str:
        return "TOOL_CALL"

    def can_handle(self, response: str) -> bool:
        return ot("TOOL_CALL") in response

    def prompt(self) -> str:
        """加载工具"""
        tools = self.get_all_tools()
        if tools:
            tools_prompt = f"<tools_section>\n"
            tools_prompt += "  <header>## 可用工具:</header>\n"
            tools_prompt += "  <tools_list>\n"
            for tool in tools:
                try:
                    tools_prompt += "    <tool>\n"
                    tools_prompt += f"      <name>名称: {tool['name']}</name>\n"
                    tools_prompt += f"      <description>描述: {tool['description']}</description>\n"
                    tools_prompt += "      <parameters>\n"
                    tools_prompt += "        <yaml>|\n"

                    # 生成格式化的YAML参数
                    yaml_params = yaml.dump(
                        tool["parameters"],
                        allow_unicode=True,
                        indent=4,
                        sort_keys=False,
                        width=120,  # 增加行宽限制
                    )

                    # 添加缩进并移除尾部空格
                    for line in yaml_params.split("\n"):
                        tools_prompt += f"          {line.rstrip()}\n"

                    tools_prompt += "        </yaml>\n"
                    tools_prompt += "      </parameters>\n"
                    tools_prompt += "    </tool>\n"

                except yaml.YAMLError as e:
                    PrettyOutput.print(
                        f"工具 {tool['name']} 参数序列化失败: {str(e)}",
                        OutputType.ERROR,
                    )
                    continue

            tools_prompt += "  </tools_list>\n"
            tools_prompt += "</tools_section>\n"
            tools_prompt += tool_call_help.rstrip()  # 移除帮助文本尾部空格
            return tools_prompt
        return ""

    def handle(self, response: str, agent_: Any) -> Tuple[bool, Any]:
        try:
            tool_call, err_msg = self._extract_tool_calls(response)
            if err_msg:
                return False, err_msg
            return False, self.handle_tool_calls(tool_call, agent_)
        except Exception as e:
            PrettyOutput.print(f"工具调用处理失败: {str(e)}", OutputType.ERROR)
            from jarvis.jarvis_agent import Agent

            agent: Agent = agent_
            return (
                False,
                f"工具调用处理失败: {str(e)}\n\n{agent.get_tool_usage_prompt()}",
            )

    def __init__(self) -> None:
        """初始化工具注册表"""
        self.tools: Dict[str, Tool] = {}
        # 加载内置工具和外部工具
        self._load_builtin_tools()
        self._load_external_tools()
        self._load_mcp_tools()
        # 应用工具配置组过滤
        self._apply_tool_config_filter()

    def _get_tool_stats(self) -> Dict[str, int]:
        """从数据目录获取工具调用统计"""
        from jarvis.jarvis_stats.stats import StatsManager
        from datetime import datetime, timedelta

        # 获取所有工具的统计数据
        tool_stats = {}
        tools = self.get_all_tools()

        # 获取所有历史数据（从很早的时间开始）
        end_time = datetime.now()
        start_time = datetime(2000, 1, 1)  # 使用一个足够早的时间

        for tool in tools:
            tool_name = tool["name"]
            # 获取该工具的统计数据
            stats_data = StatsManager.get_stats(
                metric_name=tool_name,
                start_time=start_time,
                end_time=end_time,
                tags={"group": "tool"},
            )

            # 计算总调用次数
            if stats_data and "records" in stats_data:
                total_count = sum(record["value"] for record in stats_data["records"])
                tool_stats[tool_name] = int(total_count)
            else:
                tool_stats[tool_name] = 0

        return tool_stats

    def _update_tool_stats(self, name: str) -> None:
        """更新工具调用统计"""
        from jarvis.jarvis_stats.stats import StatsManager

        StatsManager.increment(name, group="tool")

    def use_tools(self, name: List[str]) -> None:
        """使用指定工具

        参数:
            name: 要使用的工具名称列表
        """
        missing_tools = [tool_name for tool_name in name if tool_name not in self.tools]
        if missing_tools:
            PrettyOutput.print(
                f"工具 {missing_tools} 不存在，可用的工具有: {', '.join(self.tools.keys())}",
                OutputType.WARNING,
            )
        self.tools = {
            tool_name: self.tools[tool_name]
            for tool_name in name
            if tool_name in self.tools
        }

    def dont_use_tools(self, names: List[str]) -> None:
        """从注册表中移除指定工具

        参数:
            names: 要移除的工具名称列表
        """
        self.tools = {
            name: tool for name, tool in self.tools.items() if name not in names
        }

    def _apply_tool_config_filter(self) -> None:
        """应用工具配置组的过滤规则"""
        from jarvis.jarvis_utils.config import get_tool_use_list, get_tool_dont_use_list

        use_list = get_tool_use_list()
        dont_use_list = get_tool_dont_use_list()

        # 如果配置了 use 列表，只保留列表中的工具
        if use_list:
            filtered_tools = {}
            for tool_name in use_list:
                if tool_name in self.tools:
                    filtered_tools[tool_name] = self.tools[tool_name]
                else:
                    PrettyOutput.print(
                        f"警告: 配置的工具 '{tool_name}' 不存在",
                        OutputType.WARNING,
                    )
            self.tools = filtered_tools

        # 如果配置了 dont_use 列表，排除列表中的工具
        if dont_use_list:
            for tool_name in dont_use_list:
                if tool_name in self.tools:
                    del self.tools[tool_name]
                    PrettyOutput.print(
                        f"已排除工具: {tool_name}",
                        OutputType.INFO,
                    )

    def _load_mcp_tools(self) -> None:
        """加载MCP工具，优先从配置获取，其次从目录扫描"""
        from jarvis.jarvis_utils.config import get_mcp_config

        # 优先从配置获取MCP工具配置
        mcp_configs = get_mcp_config()
        if mcp_configs:
            for config in mcp_configs:
                self.register_mcp_tool_by_config(config)
            return

        # 如果配置中没有，则扫描目录
        mcp_tools_dir = Path(get_data_dir()) / "mcp"
        if not mcp_tools_dir.exists():
            return

        # 添加警告信息
        PrettyOutput.print(
            "警告: 从文件目录加载MCP工具的方式将在未来版本中废弃，请尽快迁移到JARVIS_MCP配置方式",
            OutputType.WARNING,
        )

        # 遍历目录中的所有.yaml文件
        for file_path in mcp_tools_dir.glob("*.yaml"):
            try:
                config = yaml.safe_load(open(file_path, "r", encoding="utf-8"))
                self.register_mcp_tool_by_config(config)
            except Exception as e:
                PrettyOutput.print(
                    f"文件 {file_path} 加载失败: {str(e)}", OutputType.WARNING
                )

    def _load_builtin_tools(self) -> None:
        """从内置工具目录加载工具"""
        tools_dir = Path(__file__).parent

        # 遍历目录中的所有.py文件
        for file_path in tools_dir.glob("*.py"):
            # 跳过base.py和__init__.py
            if file_path.name in ["base.py", "__init__.py", "registry.py"]:
                continue

            self.register_tool_by_file(str(file_path))

    def _load_external_tools(self) -> None:
        """从jarvis_data/tools和配置的目录加载外部工具"""
        from jarvis.jarvis_utils.config import get_central_tool_repo

        tool_dirs = [str(Path(get_data_dir()) / "tools")] + get_tool_load_dirs()

        # 如果配置了中心工具仓库，将其添加到加载路径
        central_repo = get_central_tool_repo()
        if central_repo:
            # 中心工具仓库存储在数据目录下的特定位置
            central_repo_path = os.path.join(get_data_dir(), "central_tool_repo")
            tool_dirs.append(central_repo_path)

            # 确保中心工具仓库被克隆/更新
            if not os.path.exists(central_repo_path):
                try:
                    import subprocess

                    PrettyOutput.print(
                        f"正在克隆中心工具仓库: {central_repo}", OutputType.INFO
                    )
                    subprocess.run(
                        ["git", "clone", central_repo, central_repo_path], check=True
                    )
                except Exception as e:
                    PrettyOutput.print(
                        f"克隆中心工具仓库失败: {str(e)}", OutputType.ERROR
                    )

        # --- 全局每日更新检查 ---
        daily_check_git_updates(tool_dirs, "tools")

        for tool_dir in tool_dirs:
            p_tool_dir = Path(tool_dir)
            if not p_tool_dir.exists() or not p_tool_dir.is_dir():
                continue

            # 遍历目录中的所有.py文件
            for file_path in p_tool_dir.glob("*.py"):
                # 跳过__init__.py
                if file_path.name == "__init__.py":
                    continue

                self.register_tool_by_file(str(file_path))

    def register_mcp_tool_by_config(self, config: Dict[str, Any]) -> bool:
        """从配置字典加载并注册工具

        参数:
            config: MCP工具配置字典

        返回:
            bool: 工具是否加载成功
        """
        try:
            if "type" not in config:
                PrettyOutput.print(
                    f"配置{config.get('name', '')}缺少type字段", OutputType.WARNING
                )
                return False

            # 检查enable标志
            if not config.get("enable", True):
                PrettyOutput.print(
                    f"MCP配置{config.get('name', '')}已禁用(enable=false)，跳过注册",
                    OutputType.INFO,
                )
                return False

            name = config.get("name", "mcp")

            # 注册资源工具
            def create_resource_list_func(client: McpClient):
                def execute(arguments: Dict[str, Any]) -> Dict[str, Any]:
                    args = arguments.copy()
                    args.pop("agent", None)
                    args.pop("want", None)
                    ret = client.get_resource_list()
                    PrettyOutput.print(
                        f"MCP {name} 资源列表:\n{yaml.safe_dump(ret, allow_unicode=True)}",
                        OutputType.TOOL,
                    )
                    return {
                        "success": True,
                        "stdout": yaml.safe_dump(ret, allow_unicode=True),
                        "stderr": "",
                    }

                return execute

            def create_resource_get_func(client: McpClient):
                def execute(arguments: Dict[str, Any]) -> Dict[str, Any]:
                    args = arguments.copy()
                    args.pop("agent", None)
                    args.pop("want", None)
                    if "uri" not in args:
                        return {
                            "success": False,
                            "stdout": "",
                            "stderr": "缺少必需的uri参数",
                        }
                    ret = client.get_resource(args["uri"])
                    PrettyOutput.print(
                        f"MCP {name} 获取资源:\n{yaml.safe_dump(ret, allow_unicode=True)}",
                        OutputType.TOOL,
                    )
                    return ret

                return execute

            def create_mcp_execute_func(tool_name: str, client: McpClient):
                def execute(arguments: Dict[str, Any]) -> Dict[str, Any]:
                    args = arguments.copy()
                    args.pop("agent", None)
                    args.pop("want", None)
                    ret = client.execute(tool_name, args)
                    PrettyOutput.print(
                        f"MCP {name} {tool_name} 执行结果:\n{yaml.safe_dump(ret, allow_unicode=True)}",
                        OutputType.TOOL,
                    )
                    return ret

                return execute

            if config["type"] == "stdio":
                if "command" not in config:
                    PrettyOutput.print(
                        f"配置{config.get('name', '')}缺少command字段",
                        OutputType.WARNING,
                    )
                    return False
            elif config["type"] == "sse":
                if "base_url" not in config:
                    PrettyOutput.print(
                        f"配置{config.get('name', '')}缺少base_url字段",
                        OutputType.WARNING,
                    )
                    return False
            elif config["type"] == "streamable":
                if "base_url" not in config:
                    PrettyOutput.print(
                        f"配置{config.get('name', '')}缺少base_url字段",
                        OutputType.WARNING,
                    )
                    return False
            else:
                PrettyOutput.print(
                    f"不支持的MCP客户端类型: {config['type']}", OutputType.WARNING
                )
                return False

            # 创建MCP客户端
            mcp_client: McpClient
            if config["type"] == "stdio":
                mcp_client = StdioMcpClient(config)
            elif config["type"] == "sse":
                mcp_client = SSEMcpClient(config)
            elif config["type"] == "streamable":
                mcp_client = StreamableMcpClient(config)
            else:
                raise ValueError(f"不支持的MCP客户端类型: {config['type']}")

            # 获取工具信息
            tools = mcp_client.get_tool_list()
            if not tools:
                PrettyOutput.print(
                    f"从配置{config.get('name', '')}获取工具列表失败",
                    OutputType.WARNING,
                )
                return False

            # 注册每个工具
            for tool in tools:
                # 注册工具
                self.register_tool(
                    name=f"{name}.tool_call.{tool['name']}",
                    description=tool["description"],
                    parameters=tool["parameters"],
                    func=create_mcp_execute_func(tool["name"], mcp_client),
                )

            # 注册资源列表工具
            self.register_tool(
                name=f"{name}.resource.get_resource_list",
                description=f"获取{name}MCP服务器上的资源列表",
                parameters={"type": "object", "properties": {}, "required": []},
                func=create_resource_list_func(mcp_client),
            )

            # 注册获取资源工具
            self.register_tool(
                name=f"{name}.resource.get_resource",
                description=f"获取{name}MCP服务器上的指定资源",
                parameters={
                    "type": "object",
                    "properties": {
                        "uri": {"type": "string", "description": "资源的URI标识符"}
                    },
                    "required": ["uri"],
                },
                func=create_resource_get_func(mcp_client),
            )

            return True

        except Exception as e:
            PrettyOutput.print(
                f"MCP配置{config.get('name', '')}加载失败: {str(e)}", OutputType.WARNING
            )
            return False

    def register_tool_by_file(self, file_path: str) -> bool:
        """从指定文件加载并注册工具

        参数:
            file_path: 工具文件的路径

        返回:
            bool: 工具是否加载成功
        """
        try:
            p_file_path = Path(file_path).resolve()  # 获取绝对路径
            if not p_file_path.exists() or not p_file_path.is_file():
                PrettyOutput.print(f"文件不存在: {p_file_path}", OutputType.ERROR)
                return False

            # 临时将父目录添加到sys.path
            parent_dir = str(p_file_path.parent)
            sys.path.insert(0, parent_dir)

            try:
                # 使用标准导入机制导入模块
                module_name = p_file_path.stem
                module = __import__(module_name)

                # 在模块中查找工具类
                tool_found = False
                for item_name in dir(module):
                    item = getattr(module, item_name)
                    # 检查是否是类并具有必要属性
                    if (
                        isinstance(item, type)
                        and hasattr(item, "name")
                        and hasattr(item, "description")
                        and hasattr(item, "parameters")
                        and hasattr(item, "execute")
                        and item.name == module_name
                    ):
                        if hasattr(item, "check"):
                            if not item.check():
                                continue

                        # 实例化工具类
                        tool_instance = item()

                        # 注册工具
                        self.register_tool(
                            name=tool_instance.name,
                            description=tool_instance.description,
                            parameters=tool_instance.parameters,
                            func=tool_instance.execute,
                        )
                        tool_found = True
                        break

                if not tool_found:
                    return False

                return True

            finally:
                # 从sys.path中移除目录
                sys.path.remove(parent_dir)

        except Exception as e:
            PrettyOutput.print(
                f"从 {Path(file_path).name} 加载工具失败: {str(e)}", OutputType.ERROR
            )
            return False

    @staticmethod
    def _has_tool_calls_block(content: str) -> bool:
        """从内容中提取工具调用块"""
        return (
            re.search(ot("TOOL_CALL") + r"(.*?)" + ct("TOOL_CALL"), content, re.DOTALL)
            is not None
        )

    @staticmethod
    def _extract_tool_calls(content: str) -> Tuple[Dict[str, Dict[str, Any]], str]:
        """从内容中提取工具调用。

        参数:
            content: 包含工具调用的内容

        返回:
            List[Dict]: 包含名称和参数的提取工具调用列表

        异常:
            Exception: 如果工具调用缺少必要字段
        """
        # 将内容拆分为行
        data = re.findall(
            ot("TOOL_CALL") + r"(.*?)" + ct("TOOL_CALL"), content, re.DOTALL
        )
        if not data:
            # can_handle 确保 ot("TOOL_CALL") 在内容中。
            # 如果数据为空，则表示 ct("TOOL_CALL") 可能丢失。
            if ot("TOOL_CALL") in content and ct("TOOL_CALL") not in content:
                # 尝试通过附加结束标签来修复它
                fixed_content = content.strip() + f"\n{ct('TOOL_CALL')}"

                # 再次提取，并检查YAML是否有效
                temp_data = re.findall(
                    ot("TOOL_CALL") + r"(.*?)" + ct("TOOL_CALL"),
                    fixed_content,
                    re.DOTALL,
                )

                if temp_data:
                    try:
                        yaml.safe_load(temp_data[0])  # Check if valid YAML

                        # Ask user for confirmation
                        PrettyOutput.print(
                            f"检测到缺失的 {ct('TOOL_CALL')} 标签，已自动修复。修复后的内容如下:",
                            OutputType.INFO,
                        )
                        PrettyOutput.print(fixed_content, OutputType.TOOL)
                        data = temp_data
                    except (yaml.YAMLError, EOFError, KeyboardInterrupt):
                        # Even after fixing, it's not valid YAML, or user cancelled.
                        # Fall through to the original error.
                        pass

            if not data:
                return (
                    {},
                    f"只有{ot('TOOL_CALL')}标签，未找到{ct('TOOL_CALL')}标签，调用格式错误，请检查工具调用格式。\n{tool_call_help}",
                )
        ret = []
        for item in data:
            try:
                msg = yaml.safe_load(item)
            except Exception as e:
                return (
                    {},
                    f"""yaml 解析失败，请检查工具调用格式。
                    {e}

                {tool_call_help}""",
                )

            if "name" in msg and "arguments" in msg and "want" in msg:
                ret.append(msg)
            else:
                return (
                    {},
                    f"""工具调用格式错误，请检查工具调用格式（缺少name、arguments、want字段）。

                {tool_call_help}""",
                )
        if len(ret) > 1:
            return {}, "检测到多个工具调用，请一次只处理一个工具调用。"
        return ret[0] if ret else {}, ""

    def register_tool(
        self,
        name: str,
        description: str,
        parameters: Any,
        func: Callable[..., Dict[str, Any]],
    ) -> None:
        """注册新工具

        参数:
            name: 工具名称
            description: 工具描述
            parameters: 工具参数定义
            func: 工具执行函数
        """
        if name in self.tools:
            PrettyOutput.print(
                f"警告: 工具 '{name}' 已存在，将被覆盖", OutputType.WARNING
            )
        self.tools[name] = Tool(name, description, parameters, func)

    def get_tool(self, name: str) -> Optional[Tool]:
        """获取工具

        参数:
            name: 工具名称

        返回:
            Optional[Tool]: 找到的工具实例，如果不存在则返回None
        """
        return self.tools.get(name)

    def get_all_tools(self) -> List[Dict[str, Any]]:
        """获取所有工具（Ollama格式定义）

        返回:
            List[Dict[str, Any]]: 包含所有工具信息的列表
        """
        return [tool.to_dict() for tool in self.tools.values()]

    def execute_tool(self, name: str, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """执行指定工具

        参数:
            name: 工具名称
            arguments: 工具参数

        返回:
            Dict[str, Any]: 包含执行结果的字典，包含success、stdout和stderr字段
        """
        tool = self.get_tool(name)
        if tool is None:
            return {
                "success": False,
                "stderr": f"工具 {name} 不存在，可用的工具有: {', '.join(self.tools.keys())}",
                "stdout": "",
            }

        # 更新工具调用统计
        self._update_tool_stats(name)

        return tool.execute(arguments)

    def _format_tool_output(self, stdout: str, stderr: str) -> str:
        """格式化工具输出为可读字符串

        Args:
            stdout: 标准输出
            stderr: 标准错误

        Returns:
            str: 格式化后的输出
        """
        output_parts = []
        if stdout:
            output_parts.append(f"<stdout>\n{stdout}\n</stdout>")
        if stderr:
            output_parts.append(f"<stderr>\n{stderr}\n</stderr>")
        output = "\n\n".join(output_parts)
        return "<无输出和错误>" if not output else output

    def _truncate_output(self, output: str) -> str:
        """截断过长的输出内容

        参数:
            output: 要截断的输出内容

        返回:
            截断后的内容，如果内容不超过60行则返回原内容
        """
        if len(output.splitlines()) > 60:
            lines = output.splitlines()
            return "\n".join(
                lines[:30] + ["\n...内容太长，已截取前后30行...\n"] + lines[-30:]
            )
        return output

    def handle_tool_calls(self, tool_call: Dict[str, Any], agent: Any) -> str:
        try:
            name = tool_call["name"]  # 确保name是str类型
            args = tool_call["arguments"]  # args已经是Dict[str, Any]
            want = tool_call["want"]
            args["agent"] = agent

            from jarvis.jarvis_agent import Agent

            agent_instance: Agent = agent

            if isinstance(args, str):
                try:
                    args = json.loads(args)
                except json.JSONDecodeError:
                    PrettyOutput.print(
                        f"工具参数格式无效: {name} {tool_call_help}", OutputType.ERROR
                    )
                    return ""

            # 执行工具调用
            result = self.execute_tool(name, args)  # 修正参数传递

            # 格式化输出
            output = self._format_tool_output(
                result["stdout"], result.get("stderr", "")
            )

            # 检查内容是否过大
            model_group = None
            if agent_instance.model:
                model_group = agent_instance.model.model_group
            is_large_content = is_context_overflow(output, model_group)

            if is_large_content:
                # 创建临时文件
                with tempfile.NamedTemporaryFile(
                    mode="w", suffix=".txt", delete=False
                ) as tmp_file:
                    output_file = tmp_file.name
                    tmp_file.write(output)
                    tmp_file.flush()

                try:
                    if (
                        agent_instance.model
                        and agent_instance.model.support_upload_files()
                    ):
                        summary = agent_instance.generate_summary()
                        agent_instance.clear_history()
                        upload_success = agent_instance.model.upload_files(
                            [output_file]
                        )
                        if upload_success:
                            # 删除args的agent键
                            args.pop("agent", None)
                            prompt = f"""
以下是之前对话的关键信息总结：

<content>
{summary}
</content>

上传的文件是以下工具执行结果：
{yaml.safe_dump({"name":name, "arguments":args, "want":want})}

请根据以上信息，继续完成任务。
"""
                            return prompt
                    # 使用上传的文件生成摘要
                    return self._truncate_output(output)
                finally:
                    # 清理临时文件
                    try:
                        os.unlink(output_file)
                    except Exception:
                        pass

            return output

        except Exception as e:
            PrettyOutput.print(f"工具执行失败：{str(e)}", OutputType.ERROR)
            return f"工具调用失败: {str(e)}"



from typing import List

from langchain.docstore.document import Document
from sentence_transformers.cross_encoder import (  # type: ignore
    CrossEncoder,
)


class Reranker:
    """
    一个重排器类，使用Cross-Encoder模型根据文档与给定查询的相关性
    对文档进行重新评分和排序。
    """

    def __init__(self, model_name: str):
        """
        初始化重排器。

        参数:
            model_name (str): 要使用的Cross-Encoder模型的名称。
        """
        print(f"🔍 正在初始化重排模型: {model_name}...")
        self.model = CrossEncoder(model_name)
        print("✅ 重排模型初始化成功。")

    def rerank(
        self, query: str, documents: List[Document], top_n: int = 5
    ) -> List[Document]:
        """
        根据文档与查询的相关性对文档列表进行重排。

        参数:
            query (str): 用户的查询。
            documents (List[Document]): 从初始搜索中检索到的文档列表。
            top_n (int): 重排后要返回的顶部文档数。

        返回:
            List[Document]: 一个已排序的最相关文档列表。
        """
        if not documents:
            return []

        # 创建 [查询, 文档内容] 对用于评分
        pairs = [[query, doc.page_content] for doc in documents]

        # 从Cross-Encoder模型获取分数
        scores = self.model.predict(pairs)

        # 将文档与它们的分数结合并排序
        doc_with_scores = list(zip(documents, scores))
        doc_with_scores.sort(key=lambda x: x[1], reverse=True)  # type: ignore

        # 返回前N个文档
        reranked_docs = [doc for doc, score in doc_with_scores[:top_n]]

        return reranked_docs



# -*- coding: utf-8 -*-
"""
Go language-specific code review checklist.
"""

CHECKLIST = """
## Go 代码审查检查清单

### 错误处理
- [ ] 是否正确检查和处理错误
- [ ] 是否遵循 Go 的错误处理惯例 (errors.Is, errors.As)
- [ ] 错误是否有足够的上下文信息

### 并发
- [ ] goroutine 是否会正常终止
- [ ] 是否存在竞态条件
- [ ] channel 使用是否正确
- [ ] 是否正确使用 sync 包
- [ ] 是否使用了上下文 (context) 进行超时控制

### 资源管理
- [ ] defer 是否正确使用
- [ ] 资源是否被正确关闭
- [ ] 是否有潜在的资源泄漏

### 性能
- [ ] 是否避免不必要的内存分配
- [ ] 切片和映射的容量是否合理
- [ ] 是否使用了高效的 IO 操作
- [ ] 大型结构体是否使用指针传递

### 代码风格与最佳实践
- [ ] 是否遵循官方 Go 代码风格
- [ ] 命名是否遵循 Go 的约定
- [ ] 是否避免了不必要的代码重复
- [ ] 是否正确使用接口
- [ ] 是否过度使用全局变量
- [ ] 包的设计是否合理

### 测试
- [ ] 是否有足够的单元测试
- [ ] 测试覆盖率是否合理
- [ ] 表格驱动测试是否得当

### 安全
- [ ] SQL 注入风险
- [ ] 命令注入风险
- [ ] 敏感信息是否安全存储

### 文档
- [ ] 导出的函数/类型是否有文档注释
- [ ] 复杂逻辑是否有注释说明
"""



# -*- coding: utf-8 -*-
import os
import re
import subprocess
import sys
import tempfile
from typing import Any, Dict, Optional

import typer
import yaml  # type: ignore

from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_utils.config import get_git_commit_prompt
from jarvis.jarvis_utils.git_utils import (
    confirm_add_new_files,
    find_git_root_and_cd,
    has_uncommitted_changes,
)
from jarvis.jarvis_utils.globals import get_agent, current_agent_name
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot
from jarvis.jarvis_utils.utils import init_env, is_context_overflow

app = typer.Typer(help="Git提交工具")


class GitCommitTool:
    name = "git_commit_agent"
    description = "根据代码变更自动生成并执行Git提交"
    labels = ["git", "version_control"]
    parameters = {
        "type": "object",
        "properties": {
            "root_dir": {
                "type": "string",
                "description": "Git仓库的根目录路径（可选）",
                "default": ".",
            },
            "prefix": {
                "type": "string",
                "description": "提交信息前缀（可选）",
                "default": "",
            },
            "suffix": {
                "type": "string",
                "description": "提交信息后缀（可选）",
                "default": "",
            },
        },
        "required": [],
    }

    def _extract_commit_message(self, message) -> Optional[str]:
        """Raw extraction preserving all characters"""
        r = re.search(
            r"(?i)" + ot("COMMIT_MESSAGE") + r"\s*([\s\S]*?)\s*" + ct("COMMIT_MESSAGE"),
            message,
        )
        if r:
            # 直接返回原始内容，仅去除外围空白
            return r.group(1).strip()
        return None

    def _get_last_commit_hash(self) -> str:
        process = subprocess.Popen(
            ["git", "log", "-1", "--pretty=%H"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        stdout, _ = process.communicate()
        return stdout.decode().strip()

    def _prepare_git_environment(self, root_dir: str) -> Optional[str]:
        """Prepare git environment by changing directory and checking for changes"""
        original_dir = os.getcwd()
        os.chdir(root_dir)
        find_git_root_and_cd()
        if not has_uncommitted_changes():
            PrettyOutput.print("没有未提交的更改", OutputType.SUCCESS)
            return None
        return original_dir

    def _stage_changes(self) -> None:
        """Stage all changes for commit"""
        print("📁 正在添加文件到提交...")
        subprocess.Popen(
            ["git", "add", "."], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
        ).wait()
        print("✅ 添加文件到提交")

    def execute(self, args: Dict) -> Dict[str, Any]:
        """Execute automatic commit process with support for multi-line messages and special characters"""
        try:
            original_dir = os.getcwd()
            root_dir = args.get("root_dir", ".")
            prefix = args.get("prefix", "")
            suffix = args.get("suffix", "")

            # Prepare git environment
            result = self._prepare_git_environment(root_dir)
            if result is None:
                return {"success": True, "stdout": "No changes to commit", "stderr": ""}
            original_dir = result

            confirm_add_new_files()

            if not has_uncommitted_changes():
                return {"success": True, "stdout": "No changes to commit", "stderr": ""}

            print("🚀 正在初始化提交流程...")
            self._stage_changes()

            # 获取差异
            print("📊 正在获取代码差异...")
            # 获取文件列表
            files_cmd = ["git", "diff", "--cached", "--name-only"]
            process = subprocess.Popen(
                files_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
            files_output = process.communicate()[0].decode()
            files = [f.strip() for f in files_output.split("\n") if f.strip()]
            file_count = len(files)

            # 获取完整差异
            process = subprocess.Popen(
                ["git", "diff", "--cached", "--exit-code"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            diff = process.communicate()[0].decode(errors="ignore")
            print(f"✅ 获取差异 ({file_count} 个文件)")
            try:
                temp_diff_file_path = None
                # 生成提交信息
                print("✨ 正在生成提交消息...")

                # 准备提示信息
                custom_prompt = get_git_commit_prompt()
                base_prompt = (
                    custom_prompt
                    if custom_prompt
                    else f"""根据代码差异生成提交信息：
                提交信息应使用中文书写
# 格式模板
必须使用以下格式：

<类型>(<范围>): <主题>

[可选] 详细描述变更内容和原因

# 格式规则
1. 类型: fix(修复bug), feat(新功能), docs(文档), style(格式), refactor(重构), test(测试), chore(其他)
2. 范围表示变更的模块或组件 (例如: auth, database, ui)
3. 主题行不超过72个字符，不以句号结尾，使用祈使语气
4. 如有详细描述，使用空行分隔主题和详细描述
5. 详细描述部分应解释"是什么"和"为什么"，而非"如何"
6. 仅输出提交信息，不要输出其他内容
"""
                )
                base_prompt += f"""
# 输出格式
{ot("COMMIT_MESSAGE")}
commit信息
{ct("COMMIT_MESSAGE")}
                """

                # 获取模型并尝试上传文件
                agent = get_agent(current_agent_name)
                if agent:
                    platform = agent.model
                    model_group = agent.model.model_group
                else:
                    platform = PlatformRegistry().get_normal_platform()
                    model_group = None
                upload_success = False

                # Check if content is too large
                is_large_content = is_context_overflow(diff, model_group)

                if is_large_content:
                    if not platform.support_upload_files():
                        print("❌ 差异文件太大，无法处理")
                        return {
                            "success": False,
                            "stdout": "",
                            "stderr": "错误：差异文件太大，无法处理",
                        }
                    print("📤 正在上传代码差异文件...")
                    # 创建临时文件并写入差异内容
                    with tempfile.NamedTemporaryFile(
                        mode="w", suffix=".diff", delete=False
                    ) as temp_diff_file:
                        temp_diff_file_path = temp_diff_file.name
                        temp_diff_file.write(diff)
                        temp_diff_file.flush()
                        print(f"✅ 差异内容已写入临时文件")
                    upload_success = platform.upload_files([temp_diff_file_path])
                    if upload_success:
                        print("✅ 成功上传代码差异文件")
                    else:
                        print("❌ 上传代码差异文件失败")
                        return {
                            "success": False,
                            "stdout": "",
                            "stderr": "错误：上传代码差异文件失败",
                        }
                # 根据上传状态准备完整的提示
                if is_large_content:
                    # 尝试生成提交信息
                    print("✨ 正在生成提交消息...")
                    # 使用上传的文件
                    prompt = (
                        base_prompt
                        + f"""
# 变更概述
- 变更文件数量: {file_count} 个文件
- 已上传包含完整代码差异的文件

请详细分析已上传的代码差异文件，生成符合上述格式的提交信息。
"""
                    )
                    commit_message = platform.chat_until_success(prompt)
                else:
                    prompt = (
                        base_prompt
                        + f"""
# 分析材料
{diff}
"""
                    )
                    commit_message = platform.chat_until_success(prompt)

                while True:
                    # 只在特定情况下重新获取commit_message
                    if (
                        not upload_success
                        and not is_large_content
                        and not commit_message
                    ):
                        commit_message = platform.chat_until_success(prompt)
                    extracted_message = self._extract_commit_message(commit_message)
                    # 如果成功提取，就跳出循环
                    if extracted_message:
                        commit_message = extracted_message
                        # 应用prefix和suffix
                        if prefix:
                            commit_message = f"{prefix} {commit_message}"
                        if suffix:
                            commit_message = f"{commit_message}\n{suffix}"
                        break
                    prompt = f"""格式错误，请按照以下格式重新生成提交信息：
                    {ot("COMMIT_MESSAGE")}
                    commit信息
                    {ct("COMMIT_MESSAGE")}
                    """
                    commit_message = platform.chat_until_success(prompt)
                print("✅ 生成提交消息")

                # 执行提交
                print("⚙️ 正在准备提交...")
                # Windows 兼容性：使用 delete=False 避免权限错误
                tmp_file = tempfile.NamedTemporaryFile(mode="w", delete=False)
                tmp_file_path = tmp_file.name
                try:
                    tmp_file.write(commit_message)
                    tmp_file.close()  # Windows 需要先关闭文件才能被其他进程读取

                    print("💾 正在执行提交...")
                    commit_cmd = ["git", "commit", "-F", tmp_file_path]
                    process = subprocess.Popen(
                        commit_cmd,
                        stdout=subprocess.PIPE,
                        stderr=subprocess.PIPE,
                        text=True,
                    )
                    stdout, stderr = process.communicate()

                    if process.returncode != 0:
                        # 如果提交失败，重置暂存区
                        subprocess.run(["git", "reset", "HEAD"], check=False)
                        error_msg = (
                            stderr.strip() if stderr else "Unknown git commit error"
                        )
                        raise Exception(f"Git commit failed: {error_msg}")

                    print("✅ 提交")
                finally:
                    # 手动删除临时文件
                    try:
                        os.unlink(tmp_file_path)
                    except Exception:
                        pass

                commit_hash = self._get_last_commit_hash()
                print("✅ 完成提交")
            finally:
                # 清理临时差异文件
                if temp_diff_file_path is not None and os.path.exists(
                    temp_diff_file_path
                ):
                    try:
                        os.unlink(temp_diff_file_path)
                    except Exception as e:
                        print(f"⚠️ 无法删除临时文件: {str(e)}")

            PrettyOutput.print(
                f"提交哈希: {commit_hash}\n提交消息: {commit_message}",
                OutputType.SUCCESS,
            )

            return {
                "success": True,
                "stdout": yaml.safe_dump(
                    {"commit_hash": commit_hash, "commit_message": commit_message},
                    allow_unicode=True,
                ),
                "stderr": "",
            }
        except Exception as e:
            PrettyOutput.print(f"提交失败: {str(e)}", OutputType.ERROR)
            return {
                "success": False,
                "stdout": "",
                "stderr": f"Commit failed: {str(e)}",
            }
        finally:
            # Always restore original directory
            os.chdir(original_dir)


@app.command()
def cli(
    root_dir: str = typer.Option(
        ".", "--root-dir", help="Git仓库的根目录路径"
    ),
    prefix: str = typer.Option(
        "",
        "--prefix",
        help="提交信息前缀（用空格分隔）",
    ),
    suffix: str = typer.Option(
        "",
        "--suffix",
        help="提交信息后缀（用换行分隔）",
    ),
):
    init_env("欢迎使用 Jarvis-GitCommitTool，您的Git提交助手已准备就绪！")
    tool = GitCommitTool()
    tool.execute(
        {
            "root_dir": root_dir,
            "prefix": prefix,
            "suffix": suffix,
        }
    )


def main():
    """Application entry point"""
    app()


if __name__ == "__main__":
    main()



"""
统计管理模块

提供统计数据的增加、查看、分析等功能的主接口
"""

from datetime import datetime, timedelta
from typing import Dict, List, Optional, Union, Any

from jarvis.jarvis_stats.storage import StatsStorage
from jarvis.jarvis_stats.visualizer import StatsVisualizer


class StatsManager:
    """统计管理器"""

    # 类级别的存储和可视化器实例
    _storage: Optional[StatsStorage] = None
    _visualizer: Optional[StatsVisualizer] = None

    @classmethod
    def _get_storage(cls) -> StatsStorage:
        """获取存储实例"""
        if cls._storage is None:
            cls._storage = StatsStorage()
        return cls._storage

    @classmethod
    def _get_visualizer(cls) -> StatsVisualizer:
        """获取可视化器实例"""
        if cls._visualizer is None:
            cls._visualizer = StatsVisualizer()
        return cls._visualizer

    def __init__(self, storage_dir: Optional[str] = None):
        """
        初始化统计管理器（保留以兼容旧代码）

        Args:
            storage_dir: 存储目录路径
        """
        # 如果提供了特定的存储目录，则重新初始化存储
        if storage_dir is not None:
            StatsManager._storage = StatsStorage(storage_dir)

    @staticmethod
    def increment(
        metric_name: str,
        amount: Union[int, float] = 1,
        tags: Optional[Dict[str, str]] = None,
        group: Optional[str] = None,
        unit: str = "count",
    ):
        """
        增加计数型指标

        Args:
            metric_name: 指标名称
            amount: 增加的数量，默认为1
            tags: 标签字典
            group: 指标分组，会自动添加到 tags 中
            unit: 计量单位，默认为 "count"

        Examples:
            >>> StatsManager.increment("page_views")
            >>> StatsManager.increment("downloads", 5)
            >>> StatsManager.increment("response_time", 0.123, unit="seconds")
            >>> StatsManager.increment("execute_script", 1, group="tool")
        """
        # 如果指定了分组，自动添加到 tags 中
        if group:
            if tags is None:
                tags = {}
            tags["group"] = group

        storage = StatsManager._get_storage()
        storage.add_metric(
            metric_name=metric_name,
            value=float(amount),
            unit=unit,
            timestamp=datetime.now(),
            tags=tags,
        )

    @staticmethod
    def list_metrics() -> List[str]:
        """
        列出所有指标

        Returns:
            指标名称列表
        """
        storage = StatsManager._get_storage()
        return storage.list_metrics()

    @staticmethod
    def show(
        metric_name: Optional[str] = None,
        last_hours: Optional[int] = None,
        last_days: Optional[int] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        format: str = "table",
        aggregation: str = "hourly",
        tags: Optional[Dict[str, str]] = None,
    ):
        """
        显示统计数据

        Args:
            metric_name: 指标名称，如果不指定则显示所有指标摘要
            last_hours: 最近N小时
            last_days: 最近N天
            start_time: 开始时间
            end_time: 结束时间
            format: 显示格式 (table, chart, summary)
            aggregation: 聚合方式 (hourly, daily)
            tags: 过滤标签

        Examples:
            >>> StatsManager.show()  # 显示所有指标摘要
            >>> StatsManager.show("api_calls", last_hours=24)  # 显示最近24小时
            >>> StatsManager.show("response_time", last_days=7, format="chart")  # 图表显示
        """
        # 处理时间范围
        if end_time is None:
            end_time = datetime.now()

        if start_time is None:
            if last_hours:
                start_time = end_time - timedelta(hours=last_hours)
            elif last_days:
                start_time = end_time - timedelta(days=last_days)
            else:
                start_time = end_time - timedelta(days=7)  # 默认7天

        if metric_name is None:
            # 显示所有指标摘要
            StatsManager._show_metrics_summary(start_time, end_time, tags)
        else:
            # 根据格式显示数据
            if format == "chart":
                StatsManager._show_chart(
                    metric_name, start_time, end_time, aggregation, tags
                )
            elif format == "summary":
                StatsManager._show_summary(
                    metric_name, start_time, end_time, aggregation, tags
                )
            else:
                StatsManager._show_table(metric_name, start_time, end_time, tags)

    @staticmethod
    def plot(
        metric_name: Optional[str] = None,
        last_hours: Optional[int] = None,
        last_days: Optional[int] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        aggregation: str = "hourly",
        tags: Optional[Dict[str, str]] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
    ):
        """
        绘制指标的折线图

        Args:
            metric_name: 指标名称（可选，不指定则根据标签过滤所有匹配的指标）
            last_hours: 最近N小时
            last_days: 最近N天
            start_time: 开始时间
            end_time: 结束时间
            aggregation: 聚合方式
            tags: 过滤标签
            width: 图表宽度
            height: 图表高度

        Examples:
            >>> StatsManager.plot("response_time", last_hours=24)
            >>> StatsManager.plot(tags={"service": "api"}, last_days=7)
        """
        # 处理时间范围
        if end_time is None:
            end_time = datetime.now()

        if start_time is None:
            if last_hours:
                start_time = end_time - timedelta(hours=last_hours)
            elif last_days:
                start_time = end_time - timedelta(days=last_days)
            else:
                start_time = end_time - timedelta(days=7)

        # 如果指定了metric_name，显示单个图表
        if metric_name:
            StatsManager._show_chart(
                metric_name, start_time, end_time, aggregation, tags, width, height
            )
        else:
            # 如果没有指定metric_name，根据标签过滤获取所有匹配的指标
            StatsManager._show_multiple_charts(
                start_time, end_time, aggregation, tags, width, height
            )

    @staticmethod
    def get_stats(
        metric_name: str,
        last_hours: Optional[int] = None,
        last_days: Optional[int] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        aggregation: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        """
        获取统计数据

        Args:
            metric_name: 指标名称
            last_hours: 最近N小时
            last_days: 最近N天
            start_time: 开始时间
            end_time: 结束时间
            aggregation: 聚合方式，如果指定则返回聚合数据
            tags: 过滤标签

        Returns:
            统计数据字典
        """
        # 处理时间范围
        if end_time is None:
            end_time = datetime.now()

        if start_time is None:
            if last_hours:
                start_time = end_time - timedelta(hours=last_hours)
            elif last_days:
                start_time = end_time - timedelta(days=last_days)
            else:
                start_time = end_time - timedelta(days=7)

        storage = StatsManager._get_storage()
        if aggregation:
            # 返回聚合数据
            return storage.aggregate_metrics(
                metric_name, start_time, end_time, aggregation, tags
            )
        else:
            # 返回原始数据
            records = storage.get_metrics(metric_name, start_time, end_time, tags)
            return {
                "metric": metric_name,
                "records": records,
                "count": len(records),
                "start_time": start_time.isoformat(),
                "end_time": end_time.isoformat(),
            }

    @staticmethod
    def clean_old_data(days_to_keep: int = 30):
        """
        清理旧数据

        Args:
            days_to_keep: 保留最近N天的数据
        """
        storage = StatsManager._get_storage()
        storage.delete_old_data(days_to_keep)
        print(f"已清理 {days_to_keep} 天前的数据")

    @staticmethod
    def remove_metric(metric_name: str) -> bool:
        """
        删除指定的指标及其所有数据
        
        Args:
            metric_name: 要删除的指标名称
            
        Returns:
            True 如果成功删除，False 如果指标不存在
        """
        storage = StatsManager._get_storage()
        return storage.delete_metric(metric_name)

    @staticmethod
    def _show_metrics_summary(
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        """显示所有指标摘要"""
        from rich.console import Console
        from rich.table import Table

        console = Console()
        storage = StatsManager._get_storage()
        metrics = storage.list_metrics()

        if not metrics:
            console.print("[yellow]没有找到任何统计指标[/yellow]")
            return

        # 如果没有指定时间范围，使用默认值
        if end_time is None:
            end_time = datetime.now()
        if start_time is None:
            start_time = end_time - timedelta(days=7)

        # 创建表格
        table = Table(title="统计指标摘要")
        table.add_column("指标名称", style="cyan")
        table.add_column("单位", style="green")
        table.add_column("最后更新", style="yellow")
        table.add_column("7天数据点", style="magenta")

        # 过滤满足标签条件的指标
        displayed_count = 0
        for metric in metrics:
            # 获取该指标的记录
            records = storage.get_metrics(metric, start_time, end_time, tags)
            
            # 如果指定了标签过滤，但没有匹配的记录，跳过该指标
            if tags and len(records) == 0:
                continue
            
            info = storage.get_metric_info(metric)
            unit = "-"
            last_updated = "-"
            
            if info:
                unit = info.get("unit", "-")
                last_updated = info.get("last_updated", "-")

                # 格式化时间
                if last_updated != "-":
                    try:
                        dt = datetime.fromisoformat(last_updated)
                        last_updated = dt.strftime("%Y-%m-%d %H:%M")
                    except:
                        pass

            count = len(records)
            table.add_row(metric, unit, last_updated, str(count))
            displayed_count += 1

        if displayed_count == 0:
            console.print("[yellow]没有找到符合条件的指标[/yellow]")
            if tags:
                console.print(f"过滤条件: {tags}")
        else:
            console.print(table)
            console.print(f"\n[green]总计: {displayed_count} 个指标[/green]")
            if tags:
                console.print(f"过滤条件: {tags}")

    @staticmethod
    def _show_table(
        metric_name: str,
        start_time: datetime,
        end_time: datetime,
        tags: Optional[Dict[str, str]],
    ):
        """以表格形式显示数据"""
        storage = StatsManager._get_storage()
        visualizer = StatsManager._get_visualizer()
        records = storage.get_metrics(metric_name, start_time, end_time, tags)

        # 获取指标信息
        info = storage.get_metric_info(metric_name)
        unit = info.get("unit", "") if info else ""

        # 使用visualizer显示表格
        visualizer.show_table(
            records=records,
            metric_name=metric_name,
            unit=unit,
            start_time=start_time.strftime("%Y-%m-%d %H:%M"),
            end_time=end_time.strftime("%Y-%m-%d %H:%M"),
            tags_filter=tags,
        )

    @staticmethod
    def _show_chart(
        metric_name: str,
        start_time: datetime,
        end_time: datetime,
        aggregation: str,
        tags: Optional[Dict[str, str]],
        width: Optional[int] = None,
        height: Optional[int] = None,
    ):
        """显示图表"""
        storage = StatsManager._get_storage()
        visualizer = StatsManager._get_visualizer()

        # 获取聚合数据
        aggregated = storage.aggregate_metrics(
            metric_name, start_time, end_time, aggregation, tags
        )

        if not aggregated:
            print(f"没有找到指标 '{metric_name}' 的数据")
            return

        # 获取指标信息
        info = storage.get_metric_info(metric_name)
        unit = info.get("unit", "") if info else ""

        # 准备数据
        first_item = next(iter(aggregated.values()), None)
        is_simple_count = (
            first_item
            and first_item.get("min") == 1
            and first_item.get("max") == 1
            and first_item.get("avg") == 1
        )

        if unit == "count" or is_simple_count:
            # 对于计数类指标，使用总和更有意义
            data = {k: v["sum"] for k, v in aggregated.items()}
        else:
            # 对于其他指标（如耗时），使用平均值
            data = {k: v["avg"] for k, v in aggregated.items()}  # 设置可视化器尺寸
        if width or height:
            visualizer.width = width or visualizer.width
            visualizer.height = height or visualizer.height

        # 绘制图表
        chart = visualizer.plot_line_chart(
            data=data,
            title=f"{metric_name} - {aggregation}聚合",
            unit=unit,
            show_values=True,
        )

        print(chart)

        # 显示时间范围
        from rich.panel import Panel
        from rich.console import Console

        console = Console()
        console.print(
            Panel(
                f"[cyan]{start_time.strftime('%Y-%m-%d %H:%M')}[/] ~ [cyan]{end_time.strftime('%Y-%m-%d %H:%M')}[/]",
                title="[bold]时间范围[/bold]",
                expand=False,
                style="dim",
                border_style="green",
            )
        )

    @staticmethod
    def _show_multiple_charts(
        start_time: datetime,
        end_time: datetime,
        aggregation: str,
        tags: Optional[Dict[str, str]],
        width: Optional[int] = None,
        height: Optional[int] = None,
    ):
        """根据标签过滤显示多个指标的图表"""
        from rich.console import Console

        console = Console()
        storage = StatsManager._get_storage()

        # 获取所有指标
        all_metrics = StatsManager.list_metrics()

        # 根据标签过滤指标
        matched_metrics = []
        for metric in all_metrics:
            # 获取该指标在时间范围内的数据
            records = storage.get_metrics(metric, start_time, end_time, tags)
            if records:  # 如果有匹配标签的数据
                matched_metrics.append(metric)

        if not matched_metrics:
            console.print("[yellow]没有找到匹配标签的指标数据[/yellow]")
            return

        console.print(f"[green]找到 {len(matched_metrics)} 个匹配的指标[/green]")

        # 为每个匹配的指标绘制图表
        for i, metric in enumerate(matched_metrics):
            if i > 0:
                console.print("\n" + "=" * 80 + "\n")  # 分隔符

            StatsManager._show_chart(
                metric, start_time, end_time, aggregation, tags, width, height
            )

    @staticmethod
    def _show_summary(
        metric_name: str,
        start_time: datetime,
        end_time: datetime,
        aggregation: str,
        tags: Optional[Dict[str, str]],
    ):
        """显示汇总信息"""
        storage = StatsManager._get_storage()
        visualizer = StatsManager._get_visualizer()

        # 获取聚合数据
        aggregated = storage.aggregate_metrics(
            metric_name, start_time, end_time, aggregation, tags
        )

        if not aggregated:
            print(f"没有找到指标 '{metric_name}' 的数据")
            return

        # 获取指标信息
        info = storage.get_metric_info(metric_name)
        unit = info.get("unit", "") if info else ""

        # 显示汇总
        summary = visualizer.show_summary(aggregated, metric_name, unit, tags)
        if summary:  # 如果返回了内容才打印（兼容性）
            print(summary)

        # 显示时间范围
        from rich.panel import Panel
        from rich.console import Console

        console = Console()
        console.print(
            Panel(
                f"[cyan]{start_time.strftime('%Y-%m-%d %H:%M')}[/] ~ [cyan]{end_time.strftime('%Y-%m-%d %H:%M')}[/]",
                title="[bold]时间范围[/bold]",
                expand=False,
                style="dim",
                border_style="green",
            )
        )



# -*- coding: utf-8 -*-
"""网络搜索工具。"""
from typing import Any, Dict

import requests
from markdownify import markdownify as md  # type: ignore

# pylint: disable=import-error,missing-module-docstring
# fmt: off
from ddgs import DDGS  # type: ignore[import-not-found]
# fmt: on

from jarvis.jarvis_agent import Agent
from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_utils.http import get as http_get
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class SearchWebTool:
    """处理网络搜索的类。"""

    name = "search_web"
    description = "搜索互联网上的信息"
    parameters = {
        "type": "object",
        "properties": {"query": {"type": "string", "description": "具体的问题"}},
    }

    def _search_with_ddgs(self, query: str, agent: Agent) -> Dict[str, Any]:
        # pylint: disable=too-many-locals, broad-except
        """执行网络搜索、抓取内容并总结结果。"""
        try:
            PrettyOutput.print("▶️ 使用 DuckDuckGo 开始网页搜索...", OutputType.INFO)
            results = list(DDGS().text(query, max_results=50, page=3))

            if not results:
                return {
                    "stdout": "未找到搜索结果。",
                    "stderr": "未找到搜索结果。",
                    "success": False,
                }

            full_content = ""
            visited_urls = []
            visited_count = 0

            for r in results:
                if visited_count >= 10:
                    PrettyOutput.print("ℹ️ 已成功获取10个网页，停止抓取。", OutputType.INFO)
                    break

                url = r["href"]
                title = r.get("title", url)

                try:
                    PrettyOutput.print(
                        f"📄 ({visited_count + 1}/10) 正在抓取: {title} ({url})",
                        OutputType.INFO,
                    )
                    response = http_get(url, timeout=10.0, allow_redirects=True)
                    content = md(response.text, strip=["script", "style"])
                    if content:
                        full_content += content + "\n\n"
                        visited_urls.append(url)
                        visited_count += 1
                except requests.exceptions.HTTPError as e:
                    PrettyOutput.print(
                        f"⚠️ HTTP错误 {e.response.status_code} 访问 {url}",
                        OutputType.WARNING,
                    )
                except requests.exceptions.RequestException as e:
                    PrettyOutput.print(f"⚠️ 请求错误: {e}", OutputType.WARNING)

            if not full_content.strip():
                return {
                    "stdout": "无法从任何URL抓取有效内容。",
                    "stderr": "抓取内容失败。",
                    "success": False,
                }

            url_list_str = "\n".join(f"  - {u}" for u in visited_urls)
            PrettyOutput.print(f"🔍 已成功访问并处理以下URL:\n{url_list_str}", OutputType.INFO)

            PrettyOutput.print("🧠 正在总结内容...", OutputType.INFO)
            summary_prompt = f"请为查询“{query}”总结以下内容：\n\n{full_content}"

            if not agent.model:
                return {
                    "stdout": "",
                    "stderr": "用于总结的Agent模型未找到。",
                    "success": False,
                }

            platform_name = agent.model.platform_name()
            model_name = agent.model.name()

            model = PlatformRegistry().create_platform(platform_name)
            if not model:
                return {
                    "stdout": "",
                    "stderr": "无法创建用于总结的模型。",
                    "success": False,
                }

            model.set_model_name(model_name)
            model.set_suppress_output(False)
            summary = model.chat_until_success(summary_prompt)

            return {"stdout": summary, "stderr": "", "success": True}

        except Exception as e:
            PrettyOutput.print(f"❌ 网页搜索过程中发生错误: {e}", OutputType.ERROR)
            return {
                "stdout": "",
                "stderr": f"网页搜索过程中发生错误: {e}",
                "success": False,
            }

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """
        Executes the web search.

        If the agent's model supports a native web search, it uses it.
        Otherwise, it falls back to using DuckDuckGo Search and scraping pages.
        """
        query = args.get("query")
        agent = args.get("agent")

        if not query:
            return {"stdout": "", "stderr": "缺少查询参数。", "success": False}

        if not isinstance(agent, Agent) or not agent.model:
            return {
                "stdout": "",
                "stderr": "Agent或Agent模型未找到。",
                "success": False,
            }

        if agent.model.support_web():
            model = PlatformRegistry().create_platform(agent.model.platform_name())
            if not model:
                return {"stdout": "", "stderr": "无法创建模型。", "success": False}
            model.set_model_name(agent.model.name())
            model.set_web(True)
            model.set_suppress_output(False)
            return {
                "stdout": model.chat_until_success(query),
                "stderr": "",
                "success": True,
            }

        return self._search_with_ddgs(query, agent)

    @staticmethod
    def check() -> bool:
        """Check if the tool is available."""
        return True



# -*- coding: utf-8 -*-
import json
import os
import time
import uuid
from typing import Any, Dict, Generator, List, Tuple

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils import http
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ot, ct
from jarvis.jarvis_utils.utils import while_success


class TongyiPlatform(BasePlatform):
    """Tongyi platform implementation"""

    # Supported image formats
    IMAGE_EXTENSIONS = {".jpg", ".jpeg", ".png", ".gif", ".webp", ".bmp", ".tiff"}

    def __init__(self):
        """Initialize Tongyi platform"""
        super().__init__()
        self.session_id = ""
        self.cookies = os.getenv("TONGYI_COOKIES", "")
        self.request_id = ""
        self.msg_id = ""
        self.model_name = ""
        self.uploaded_file_info = []
        self.system_message = ""  # System message for initialization
        self.first_chat = True  # Flag for first chat

    def _get_base_headers(self):
        return {
            "Host": "api.tongyi.com",
            "Connection": "keep-alive",
            "X-Platform": "pc_tongyi",
            "sec-ch-ua-platform": "Windows",
            "sec-ch-ua": '"Chromium";v="136", "Microsoft Edge";v="136", "Not.A/Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/136.0.0.0 Safari/537.36 Edg/136.0.0.0",
            "accept": "application/json, text/plain, */*",
            "DNT": "1",
            "Content-Type": "application/json",
            "Origin": "https://www.tongyi.com",
            "Sec-Fetch-Site": "same-site",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": "https://www.tongyi.com/qianwen",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
            "Cookie": self.cookies,
        }

    def set_model_name(self, model_name: str):
        """Set model name

        Args:
            model_name: Model name to use
        """
        self.model_name = model_name

    def _generate_request_id(self):
        self.request_id = str(uuid.uuid4()).replace("-", "")

    def support_upload_files(self) -> bool:
        """Check if platform supports upload files"""
        return True

    def chat(self, message: str) -> Generator[str, None, None]:
        if not self.request_id:
            self._generate_request_id()
        url = "https://api.tongyi.com/dialog/conversation"
        headers = self._get_base_headers()

        headers["accept"] = "text/event-stream"

        # Prepare contents array with message
        contents = [
            {
                "content": message,
                "contentType": "text",
                "role": "user",
                "ext": {
                    "searchType": "depth" if self.web else "",
                    "pptGenerate": False,
                    "deepThink": self.model_name == "Thinking",
                    "deepResearch": self.model_name == "Deep-Research",
                },
            }
        ]

        # Add system message if it's first chat
        if self.first_chat and self.system_message:
            contents.insert(
                0,
                {
                    "content": self.system_message,
                    "contentType": "text",
                    "role": "system",
                    "ext": {
                        "searchType": "depth" if self.web else "",
                        "pptGenerate": False,
                        "deepThink": self.model_name == "Thinking",
                        "deepResearch": self.model_name == "Deep-Research",
                    },
                },
            )
            self.first_chat = False

        # Add uploaded files to contents if available and clear after use
        if self.uploaded_file_info:
            for file_info in self.uploaded_file_info:
                # Determine content type based on fileKey extension
                file_ext = os.path.splitext(file_info["fileKey"])[1].lower()
                is_image = file_ext in self.IMAGE_EXTENSIONS

                contents.append(
                    {
                        "role": "user",
                        "contentType": "image" if is_image else "file",
                        "content": file_info["url"],
                        "ext": {
                            "fileSize": file_info.get("fileSize", 0),
                            "batchId": file_info.get("batchId", ""),
                            "docId": file_info.get("docId", ""),
                        },
                    }
                )
            # Clear uploaded file info after using it
            self.uploaded_file_info = []

        payload: Dict[str, Any] = {
            "model": "",
            "action": "next",
            "mode": "chat",
            "userAction": "new_top",
            "requestId": self.request_id,
            "sessionId": self.session_id,
            "sessionType": "text_chat",
            "parentMsgId": self.msg_id,
            "params": {
                "agentId": "",
                "searchType": "depth" if self.web else "",
                "pptGenerate": False,
                "bizScene": "code_chat" if self.model_name == "Code-Chat" else "",
                "bizSceneInfo": {},
                "specifiedModel": "",
                "deepThink": self.model_name == "Thinking",
                "deepResearch": self.model_name == "Deep-Research",
                "fileUploadBatchId": (
                    self.uploaded_file_info[0]["batchId"]
                    if self.uploaded_file_info
                    else ""
                ),
            },
            "contents": contents,
        }

        try:
            # 使用新的stream_post接口发送消息请求，获取流式响应
            response_stream = while_success(
                lambda: http.stream_post(url, headers=headers, json=payload),
                sleep_time=5,
            )

            msg_id = ""
            session_id = ""
            thinking_content = ""
            text_content = ""
            in_thinking = False

            # 处理流式响应
            for line in response_stream:
                if not line.strip():
                    continue

                # SSE格式的行通常以"data: "开头
                if line.startswith("data: "):
                    try:
                        data = json.loads(line[6:])
                        # 记录消息ID和会话ID
                        if "msgId" in data:
                            msg_id = data["msgId"]
                        if "sessionId" in data:
                            session_id = data["sessionId"]

                        if "contents" in data and len(data["contents"]) > 0:
                            for content in data["contents"]:
                                if content.get("contentType") == "think":
                                    if not in_thinking:
                                        yield f"{ot('think')}\n\n"
                                        in_thinking = True
                                    if content.get("incremental"):
                                        tmp_content = json.loads(
                                            content.get("content")
                                        )["content"]
                                        thinking_content += tmp_content
                                        yield tmp_content
                                    else:
                                        tmp_content = json.loads(
                                            content.get("content")
                                        )["content"]
                                        if len(thinking_content) < len(tmp_content):
                                            yield tmp_content[len(thinking_content) :]
                                            thinking_content = tmp_content
                                        else:
                                            yield f"\r\n{ct('think')}\n"[
                                                len(thinking_content)
                                                - len(tmp_content) :
                                            ]
                                            thinking_content = tmp_content
                                        in_thinking = False
                                elif content.get("contentType") == "text":
                                    if in_thinking:
                                        continue
                                    if content.get("incremental"):
                                        tmp_content = content.get("content")
                                        text_content += tmp_content
                                        yield tmp_content
                                    else:
                                        tmp_content = content.get("content")
                                        if len(text_content) < len(tmp_content):
                                            yield tmp_content[len(text_content) :]
                                            text_content = tmp_content

                    except json.JSONDecodeError:
                        continue

            self.msg_id = msg_id
            self.session_id = session_id

            return

        except Exception as e:
            raise Exception(f"Chat failed: {str(e)}")

    def _get_upload_token(self) -> Dict[str, Any]:
        """Get upload token from Tongyi API

        Returns:
            Dict[str, Any]: Upload token information including accessId, bucketName, etc.
        """
        url = "https://api.tongyi.com/dialog/uploadToken"
        headers = self._get_base_headers()
        payload: Dict[str, Any] = {}

        try:
            response = while_success(
                lambda: http.post(url, headers=headers, json=payload), sleep_time=5
            )
            if response.status_code != 200:
                raise Exception(f"HTTP {response.status_code}: {response.text}")

            result = response.json()
            if not result.get("success"):
                raise Exception(f"Failed to get upload token: {result.get('errorMsg')}")

            return result.get("data", {})

        except Exception as e:
            raise Exception(f"Failed to get upload token: {str(e)}")

    def upload_files(self, file_list: List[str]) -> bool:
        """Upload files to Tongyi platform and get download links

        Args:
            file_list: List of file paths to upload

        Returns:
            List[Dict[str, str]]: List of dictionaries containing file info and download URLs
        """
        try:
            upload_token = self._get_upload_token()
            uploaded_files = []

            for file_path in file_list:
                file_name = os.path.basename(file_path)
                print(f"🔍 上传文件 {file_name}")
                try:
                    if not os.path.exists(file_path):
                        print(f"❌ 文件不存在: {file_path}")
                        return False

                    # Get file name and content type
                    content_type = self._get_content_type(file_path)

                    print(f"🔍 准备上传文件: {file_name}")

                    # Prepare form data
                    form_data = {
                        "OSSAccessKeyId": upload_token["accessId"],
                        "policy": upload_token["policy"],
                        "signature": upload_token["signature"],
                        "key": f"{upload_token['dir']}{file_name}",
                        "dir": upload_token["dir"],
                        "success_action_status": "200",
                    }

                    # Prepare files
                    files = {"file": (file_name, open(file_path, "rb"), content_type)}

                    print(f"📤 正在上传文件: {file_name}")

                    # Upload file
                    response = http.post(
                        upload_token["host"], data=form_data, files=files
                    )

                    if response.status_code != 200:
                        print(f"❌ 上传失败 {file_name}: HTTP {response.status_code}")
                        return False

                    # Determine file type based on extension
                    file_ext = os.path.splitext(file_path)[1].lower()
                    is_image = file_ext in self.IMAGE_EXTENSIONS

                    uploaded_files.append(
                        {
                            "fileKey": file_name,
                            "fileType": "image" if is_image else "file",
                            "dir": upload_token["dir"],
                        }
                    )

                    print(f"🔍 获取下载链接: {file_name}")

                    # Get download links for uploaded files
                    url = "https://api.tongyi.com/dialog/downloadLink/batch"
                    headers = self._get_base_headers()
                    payload = {
                        "fileKeys": [f["fileKey"] for f in uploaded_files],
                        "fileType": (
                            "image"
                            if any(f["fileType"] == "image" for f in uploaded_files)
                            else "file"
                        ),
                        "dir": upload_token["dir"],
                    }

                    response = http.post(url, headers=headers, json=payload)
                    if response.status_code != 200:
                        print(f"❌ 获取下载链接失败: HTTP {response.status_code}")
                        return False

                    result = response.json()
                    if not result.get("success"):
                        print(f"❌ 获取下载链接失败: {result.get('errorMsg')}")
                        return False

                    # Add files to chat
                    self.uploaded_file_info = result.get("data", {}).get("results", [])
                    for file_info in self.uploaded_file_info:
                        print(f"🔍 添加文件到对话: {file_name}")
                        add_url = "https://api.tongyi.com/assistant/api/chat/file/add"
                        add_payload = {
                            "workSource": "chat",
                            "terminal": "web",
                            "workCode": "0",
                            "channel": "home",
                            "workType": "file",
                            "module": "uploadhistory",
                            "workName": file_info["fileKey"],
                            "workId": file_info["docId"],
                            "workResourcePath": file_info["url"],
                            "sessionId": "",
                            "batchId": str(uuid.uuid4()).replace("-", "")[
                                :32
                            ],  # Generate random batchId
                            "fileSize": os.path.getsize(file_path),
                        }

                        add_response = http.post(
                            add_url, headers=headers, json=add_payload
                        )
                        if add_response.status_code != 200:
                            print(
                                f"❌ 添加文件到对话失败: HTTP {add_response.status_code}"
                            )
                            continue

                        add_result = add_response.json()
                        if not add_result.get("success"):
                            print(
                                f"❌ 添加文件到对话失败: {add_result.get('errorMsg')}"
                            )
                            continue

                        file_info.update(add_result.get("data", {}))

                    print(f"✅ 文件 {file_name} 上传成功")
                    time.sleep(1)  # 短暂暂停以便用户看到成功状态

                except Exception as e:
                    print(f"❌ 上传文件 {file_name} 时出错: {str(e)}")
                    return False
            return True

        except Exception as e:
            PrettyOutput.print(f"Error uploading files: {str(e)}", OutputType.ERROR)
            return False

    def _get_content_type(self, file_path: str) -> str:
        """Get content type for file

        Args:
            file_path: Path to file

        Returns:
            str: Content type
        """
        ext = os.path.splitext(file_path)[1].lower()
        content_types = {
            ".txt": "text/plain",
            ".md": "text/markdown",
            ".doc": "application/msword",
            ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
            ".xls": "application/vnd.ms-excel",
            ".xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            ".pdf": "application/pdf",
            ".epub": "application/epub+zip",
            ".mobi": "application/x-mobipocket-ebook",
            ".jpg": "image/jpeg",
            ".jpeg": "image/jpeg",
            ".png": "image/png",
            ".gif": "image/gif",
            ".webp": "image/webp",
            ".bmp": "image/bmp",
            ".tiff": "image/tiff",
        }
        return content_types.get(ext, "application/octet-stream")

    def name(self) -> str:
        """Get platform name

        Returns:
            str: Platform name
        """
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """Get platform name

        Returns:
            str: Platform name
        """
        return "tongyi"

    def delete_chat(self) -> bool:
        """Delete chat history

        Returns:
            bool: True if deletion successful, False otherwise
        """
        if not self.session_id:
            return True

        url = "https://api.tongyi.com/dialog/session/delete"
        headers = self._get_base_headers()
        payload: Dict[str, Any] = {"sessionId": self.session_id}

        try:
            response = while_success(
                lambda: http.post(url, headers=headers, json=payload), sleep_time=5
            )
            if response.status_code != 200:
                PrettyOutput.print(
                    f"Failed to delete chat: HTTP {response.status_code}",
                    OutputType.ERROR,
                )
                return False
            self.request_id = ""
            self.session_id = ""
            self.msg_id = ""
            self.first_chat = True  # Reset first_chat flag
            return True
        except Exception as e:
            PrettyOutput.print(f"Error deleting chat: {str(e)}", OutputType.ERROR)
            return False

    def save(self, file_path: str) -> bool:
        """Save chat session to a file."""
        if not self.session_id:
            PrettyOutput.print("没有活动的会话可供保存", OutputType.WARNING)
            return False

        state = {
            "session_id": self.session_id,
            "request_id": self.request_id,
            "msg_id": self.msg_id,
            "model_name": self.model_name,
            "uploaded_file_info": self.uploaded_file_info,
            "system_message": self.system_message,
            "first_chat": self.first_chat,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.print(f"会话已成功保存到 {file_path}", OutputType.SUCCESS)
            return True
        except Exception as e:
            PrettyOutput.print(f"保存会话失败: {str(e)}", OutputType.ERROR)
            return False

    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.session_id = state.get("session_id", "")
            self.request_id = state.get("request_id", "")
            self.msg_id = state.get("msg_id", "")
            self.model_name = state.get("model_name", "")
            self.uploaded_file_info = state.get("uploaded_file_info", [])
            self.system_message = state.get("system_message", "")
            self.first_chat = state.get("first_chat", True)
            self._saved = True

            PrettyOutput.print(f"从 {file_path} 成功恢复会话", OutputType.SUCCESS)
            return True
        except FileNotFoundError:
            PrettyOutput.print(f"会话文件未找到: {file_path}", OutputType.ERROR)
            return False
        except Exception as e:
            PrettyOutput.print(f"恢复会话失败: {str(e)}", OutputType.ERROR)
            return False

    def set_system_prompt(self, message: str):
        """Set system message

        Args:
            message: System message to set
        """
        self.system_message = message

    def get_model_list(self) -> List[Tuple[str, str]]:
        """Get available model list

        Returns:
            List[Tuple[str, str]]: List of (model_id, model_name) tuples
        """
        return [
            ("Normal", "Normal"),
            ("Thinking", "Thinking"),
            ("Deep-Research", "Deep-Research"),
            ("Code-Chat", "Code-Chat"),
        ]

    def support_web(self) -> bool:
        """Check if platform supports web functionality

        Returns:
            bool: True if web is supported, False otherwise
        """
        return True

    @classmethod
    def get_required_env_keys(cls) -> List[str]:
        """
        获取通义平台所需的环境变量键列表

        返回:
            List[str]: 环境变量键的列表
        """
        return ["TONGYI_COOKIES"]



# -*- coding: utf-8 -*-
"""
SQL language-specific code review checklist.
"""

CHECKLIST = """
## SQL 代码审查检查清单

### 查询设计和可读性
- [ ] SQL 语句是否格式化和缩进一致
- [ ] 是否使用有意义的表和列名
- [ ] 是否有注释说明复杂查询
- [ ] 是否使用了一致的命名约定
- [ ] 是否避免了过度复杂的查询
- [ ] 是否适当使用子查询和公共表表达式 (CTE)

### 性能优化
- [ ] 是否有合适的索引支持查询
- [ ] 大表查询是否使用了分页
- [ ] 是否避免了全表扫描
- [ ] WHERE 条件是否能有效利用索引
- [ ] JOIN 操作是否高效
- [ ] 是否避免了使用 SELECT *
- [ ] 是否避免了不必要的排序和聚合
- [ ] 是否使用了适当的索引提示
- [ ] 大批量操作是否分批处理
- [ ] 触发器和存储过程是否高效

### 数据完整性和约束
- [ ] 是否定义了适当的主键
- [ ] 是否使用了外键约束
- [ ] 是否添加了适当的 NOT NULL 约束
- [ ] 是否设置了合理的默认值
- [ ] 是否使用了检查约束验证数据
- [ ] 事务是否保持一致性

### 安全性
- [ ] 是否防范 SQL 注入
- [ ] 是否使用参数化查询
- [ ] 权限是否遵循最小权限原则
- [ ] 敏感数据是否加密存储
- [ ] 是否限制了查询结果集大小
- [ ] 是否避免了在查询中使用 SQL 动态构建

### 并发和锁定
- [ ] 是否考虑了并发访问
- [ ] 事务范围是否最小化
- [ ] 是否使用了适当的隔离级别
- [ ] 是否避免了长时间锁定资源
- [ ] 是否考虑了死锁问题
- [ ] 是否处理了锁超时错误

### 可维护性和兼容性
- [ ] 脚本是否是幂等的 (可重复运行)
- [ ] 是否有数据库版本控制和迁移策略
- [ ] 是否考虑了数据库引擎兼容性
- [ ] 是否有针对各种错误情况的处理
- [ ] 是否有回滚策略
- [ ] 模式更改是否安全可靠

### 特定数据库最佳实践
- [ ] 是否遵循特定数据库引擎的最佳实践
- [ ] 是否合理使用数据库特有功能
- [ ] 是否考虑了不同数据库方言的差异
- [ ] 是否使用了合适的数据类型

### 测试
- [ ] 是否有针对 SQL 的单元测试
- [ ] 是否测试了边缘情况
- [ ] 是否验证了结果集的正确性
- [ ] 是否测试了性能和负载
- [ ] 是否有数据恢复测试
"""



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Any, Dict, List


class McpClient(ABC):
    """MCP客户端抽象基类"""

    @abstractmethod
    def get_tool_list(self) -> List[Dict[str, Any]]:
        """获取工具列表

        返回:
            List[Dict[str, Any]]: 工具列表，每个工具包含以下字段：
                - name: str - 工具名称
                - description: str - 工具描述
                - parameters: Dict - 工具参数
        """
        pass

    @abstractmethod
    def execute(self, tool_name: str, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """执行工具

        参数:
            tool_name: 工具名称
            arguments: 参数字典，包含工具执行所需的参数

        返回:
            Dict[str, Any]: 执行结果，包含以下字段：
                - success: bool - 是否执行成功
                - stdout: str - 标准输出
                - stderr: str - 标准错误
        """
        pass

    @abstractmethod
    def get_resource_list(self) -> List[Dict[str, Any]]:
        """获取资源列表

        返回:
            List[Dict[str, Any]]: 资源列表，每个资源包含以下字段：
                - uri: str - 资源的唯一标识符
                - name: str - 资源的名称
                - description: str - 资源的描述（可选）
                - mimeType: str - 资源的MIME类型（可选）
        """
        pass

    @abstractmethod
    def get_resource(self, uri: str) -> Dict[str, Any]:
        """获取指定资源的内容

        参数:
            uri: str - 资源的URI标识符

        返回:
            Dict[str, Any]: 资源内容，包含以下字段：
                - uri: str - 资源的URI
                - mimeType: str - 资源的MIME类型（可选）
                - text: str - 文本内容（如果是文本资源）
                - blob: str - 二进制内容（如果是二进制资源，base64编码）
        """
        pass



# -*- coding: utf-8 -*-
import re
from typing import Any, Dict, List, Optional, Tuple, Union

import yaml

from jarvis.jarvis_agent import Agent
from jarvis.jarvis_agent.output_handler import OutputHandler
from jarvis.jarvis_tools.registry import ToolRegistry
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot


class MultiAgent(OutputHandler):
    def __init__(self, agents_config: List[Dict], main_agent_name: str):
        self.agents_config = agents_config
        self.agents_config_map = {c["name"]: c for c in agents_config}
        self.agents: Dict[str, Agent] = {}
        self.main_agent_name = main_agent_name
        self.original_question: Optional[str] = None

    def prompt(self) -> str:
        return f"""
# 多智能体消息发送

## 交互原则与策略
### 消息处理规范
- **单一操作原则**：每轮只执行一个操作（工具调用或消息发送）
- **完整性原则**：确保消息包含所有必要信息，避免歧义
- **明确性原则**：清晰表达意图、需求和期望结果
- **上下文保留**：在消息中包含足够的背景信息

### 消息格式标准
```
{ot("SEND_MESSAGE")}
to: 智能体名称    # 目标智能体名称
content: |2
  # 消息主题
  ## 背景信息
  [提供必要的上下文和背景]
  ## 具体需求
  [明确表达期望完成的任务]
  ## 相关资源
  [列出相关文档、数据或工具]
  ## 期望结果
  [描述期望的输出格式和内容]
  ## 下一步计划
  [描述下一步的计划和行动]
{ct("SEND_MESSAGE")}
```

或者：

```
{ot("SEND_MESSAGE")}
to: 智能体名称    # 目标智能体名称
content: |2
  # 消息主题
  ## 任务结果
  [任务完成结果，用于反馈]
{ct("SEND_MESSAGE")}
```

## 可用智能体资源
{chr(10).join([f"- **{c['name']}**: {c.get('description', '')}" for c in self.agents_config])}
"""

    def can_handle(self, response: str) -> bool:
        return len(self._extract_send_msg(response)) > 0

    def handle(self, response: str, agent: Any) -> Tuple[bool, Any]:
        send_messages = self._extract_send_msg(response)
        if len(send_messages) > 1:
            return (
                False,
                f"Send multiple messages, please only send one message at a time.",
            )
        if len(send_messages) == 0:
            return False, ""
        return True, send_messages[0]

    def name(self) -> str:
        return "SEND_MESSAGE"

    @staticmethod
    def _extract_send_msg(content: str) -> List[Dict]:
        """Extract send message from content.

        Args:
            content: The content containing send message
        """
        if ot("SEND_MESSAGE") in content and ct("SEND_MESSAGE") not in content:
            content += "\n" + ct("SEND_MESSAGE")
        data = re.findall(
            ot("SEND_MESSAGE") + r"\n(.*?)\n" + ct("SEND_MESSAGE"), content, re.DOTALL
        )
        ret = []
        for item in data:
            try:
                msg = yaml.safe_load(item)
                if "to" in msg and "content" in msg:
                    ret.append(msg)
            except Exception as e:
                continue
        return ret

    def _get_agent(self, name: str) -> Union[Agent, None]:
        if name in self.agents:
            return self.agents[name]

        if name not in self.agents_config_map:
            return None

        config = self.agents_config_map[name].copy()

        if name != self.main_agent_name and self.original_question:
            system_prompt = config.get("system_prompt", "")
            config[
                "system_prompt"
            ] = f"{system_prompt}\n\n# 原始问题\n{self.original_question}"

        output_handler = config.get("output_handler", [])
        if len(output_handler) == 0:
            output_handler = [
                ToolRegistry(),
                self,
            ]
        else:
            if not any(isinstance(h, MultiAgent) for h in output_handler):
                output_handler.append(self)
        config["output_handler"] = output_handler

        agent = Agent(**config)
        self.agents[name] = agent
        return agent

    def run(self, user_input: str) -> str:
        self.original_question = user_input
        last_agent_name = self.main_agent_name

        agent = self._get_agent(self.main_agent_name)
        if not agent:
            # This should not happen if main_agent_name is correctly configured
            return f"主智能体 {self.main_agent_name} 未找到"

        msg: Any = agent.run(user_input)

        while msg:
            if isinstance(msg, str):
                return msg

            if not isinstance(msg, Dict):
                # Should not happen if agent.run() returns str or Dict
                PrettyOutput.print(f"未知消息类型: {type(msg)}", OutputType.WARNING)
                break

            prompt = f"""
Please handle this message:
from: {last_agent_name}
content: {msg['content']}
"""
            to_agent_name = msg.get("to")
            if not to_agent_name:
                return f"消息中未指定 `to` 字段"

            if to_agent_name not in self.agents_config_map:
                PrettyOutput.print(
                    f"未找到智能体 {to_agent_name}，正在重试...", OutputType.WARNING
                )
                agent = self._get_agent(last_agent_name)
                if not agent:
                    return f"智能体 {last_agent_name} 未找到"
                msg = agent.run(
                    f"未找到智能体 {to_agent_name}，可用智能体列表: {list(self.agents_config_map.keys())}"
                )
                continue

            PrettyOutput.print(
                f"{last_agent_name} 正在向 {to_agent_name} 发送消息...", OutputType.INFO
            )

            agent = self._get_agent(to_agent_name)
            if not agent:
                return f"智能体 {to_agent_name} 未找到"

            last_agent_name = agent.name
            msg = agent.run(prompt)
        return ""
